/*  Project pounds

    Copyright  1994. All Rights Reserved.

    SUBSYSTEM:    pounds.apx Application
    FILE:         areadoc.cpp
    AUTHOR:       Slash


    OVERVIEW
    ========
    Source file for implementation of TAreaDocument (TFileDocument).
*/

/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  Envy Diku Mud improvements copyright (C) 1994 by Michael Quan, David   *
 *  Love, Guilherme 'Willie' Arnold, and Mitchell Tse.                     *
 *                                                                         *
 *  In order to use any part of this Envy Diku Mud, you must comply with   *
 *  the original Diku license in 'license.doc', the Merc license in        *
 *  'license.txt', as well as the Envy license in 'license.nvy'.           *
 *  In particular, you may not remove either of these copyright notices.   *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

#include <owl\owlpch.h>
#pragma hdrstop

#include <owl\inputdia.h>

#include "areadoc.h"

extern poundsApp *TheMuditorApp;

//{{TAreaDocument Implementation}}

// Reverse direction for exits
const	sh_int TAreaDocument::rev_dir[6] = { 2, 3, 0, 1, 5, 4 };

// @@@ Move into class
BOOL fFoundMOBprog;

TAreaDocument::TAreaDocument (TDocument* parent):
    TFileDocument(parent),
    TheHelps(100, 0, 10),
    TheMobiles(100, 0, 10),
    TheObjects(100, 0, 10),
    TheRooms(100, 0, 10),
    TheShops(10, 0, 5),
    TheResets(100, 0, 20),
    TheMobResets(20, 0, 10)
{
    // INSERT>> Your constructor code here.
    fIsOpen = FALSE;

    nDefaultSectorType = SECT_CITY;
    nDefaultSectorFlags = 0;

    nDefaultAction = ACT_STAY_AREA;
    nDefaultAffectedBy = 0;

    recall = -1;

    // For Troll's AreaData
  //  fUsesAreaData = TRUE;
    area_data.area_flags = 0;
    area_data.plane = 0;
 //   area_data.fCustomWeather = FALSE;
 //   static AREA_WEATHER NormalWeather = { 40, 80, 0, 100, 0, 50, 50, 50, 5 };
 //   area_data.Weather = NormalWeather;
 //   area_data.timezone = 0;
}

// Make all of these part of class
static void DeleteARoom(TRoom *& pRoom, void *)
{
   delete pRoom;
}

static void DeleteAHelp(THelpData *& pHelp, void *)
{
   delete pHelp;
}

static void DeleteAObject(TObject *& pObject, void *)
{
   delete pObject;
}

static void DeleteAMobile(TMobile *& pMobile, void *)
{
   delete pMobile;
}

static void DeleteAShop(TShopData *& pShop, void *)
{
   delete pShop;
}

static void DeleteAReset(TReset *& pReset, void *)
{
   delete pReset;
}

//static void DeleteAMobReset(TMobReset *& pMobReset, void *)
//{
//	delete pMobReset;
//}

TAreaDocument::~TAreaDocument ()
{
	 // INSERT>> Your destructor code here.
	 TheHelps.ForEach(DeleteAHelp, 0);
	 TheMobiles.ForEach(DeleteAMobile, 0);
	 TheObjects.ForEach(DeleteAObject, 0);
	 TheRooms.ForEach(DeleteARoom, 0);
	 TheShops.ForEach(DeleteAShop, 0);
	 TheResets.ForEach(DeleteAReset, 0);
//    TheMobResets.ForEach(DeleteAMobReset, 0);
}


bool TAreaDocument::IsOpen ()
{
    return fIsOpen;
}


bool TAreaDocument::Open (int /*mode*/, LPCSTR path)
{
    // I have no idea what this is for...  It's in the linedoc example
    if (path)
       SetDocPath(path);

    TheMuditorApp = MuditorApp =
       TYPESAFE_DOWNCAST(GetDocManager().GetApplication(), poundsApp);
	 if (GetDocPath()) { // Handle File|Open
		 TPointer<TInStream> is = InStream(ofRead|ofBinary);
		 if (!is)
			return FALSE;

    fIsOpen = TRUE;
    fFoundMOBprog = FALSE;
    TRY {
       nNextRoomVNum = nNextObjVNum = nNextMobVNum = -1;
		 for ( ; ; )
			 {
			 char *word;

			 if ( fread_letter( *is ) != '#' )
				 {
				 SyntaxError(*is, "Looking for new section", "# not found");
				 break;
				 }

			 word = fread_word( *is, "Reading new section name" );
          strcpy(szSection, word);
          strcpy(szLastSuccess, "<none>");

				  if ( word[0] == '$'               )                 break;
			 else if ( !stricmp( word, "AREA"     ) ) load_area    (*is);
			 else if ( !stricmp( word, "HELPS"    ) ) load_helps   (*is);
			 else if ( !stricmp( word, "MOBILES"  ) ) load_mobiles (*is, MuditorApp->CurrentMode);
			 else if ( !stricmp( word, "MOBPROGS" ) ) load_mobprogs(*is);
			 else if ( !stricmp( word, "OBJECTS"  ) ) load_objects (*is, MuditorApp->CurrentMode);
			 else if ( !stricmp( word, "RESETS"   ) ) load_resets  (*is);
			 else if ( !stricmp( word, "ROOMS"    ) ) load_rooms   (*is);
			 else if ( !stricmp( word, "SHOPS"    ) ) load_shops   (*is);
			 else if ( !stricmp( word, "SPECIALS" ) ) load_specials(*is);
          // Following is Envy
          else if ( !stricmp( word, "RECALL"   ) ) load_recall  (*is);
          // Following is Troll
          else if ( !stricmp( word, "AREADATA" ) ) load_troll_area(*is);
          // Following is ROM
          else if ( !stricmp( word, "MOBOLD"   ) ) load_mobiles (*is, poundsApp::MercMode);
          else if ( !stricmp( word, "OBJOLD"   ) ) load_objects (*is, poundsApp::MercMode);
			 else
				 {
				 SyntaxError(*is, "Looking for a section", "MZF does not understand this section type");
				 break;
				 }
			 }

       fix_exits();
       if (fFoundMOBprog && MuditorApp->fMOBprograms == FALSE)
          {
          MessageBox(0, "Warning:  You have MOBprograms, but have selected no "
             "MOBprograms in your configuration.  Your MOBprograms will be "
             "stripped when you exit unless you go into 'File|File Format' "
             "and turn on MOBprogram support", "M.Z.F.",
             MB_ICONINFORMATION|MB_OK);
          }
       }
       CATCH( (xmsg& x)
          {
          GetDocManager().GetApplication()->Error(x, 0, 0);
          return FALSE;
          }
       )
		 }
    else // Handle File|New
       {
       name = "{ 5 35} M.Z.F.  Prototype for New Area";
//       recall = ROOM_VNUM_TEMPLE;
       recall = -1;

       char szBuffer[8] = "9900";
       if (TInputDialog(GetDocManager().GetApplication()->GetMainWindow(),
          "MAKE.ZONES.FAST", "Enter beginning VNUM",
             szBuffer, sizeof szBuffer).Execute() != IDOK)
          return FALSE;

       nNextRoomVNum = nNextObjVNum = nNextMobVNum = (sh_int) atoi(szBuffer);
       fIsOpen = TRUE;
       }

    return TRUE;
}

/*
 * Snarf an 'area' header line.
 */
void TAreaDocument::load_area( istream& is )
{
    name		= fread_string( is, "Reading zone name" );
    recall  = -1;
}



/*
 * Snarf a help section.
 */
void TAreaDocument::load_helps( istream& is )
{
    PHelpData pHelp;

    for ( ; ; )
    {
    pHelp = new THelpData(this);
	pHelp->level	= (sh_int) fread_number( is, "Reading help level" );
	pHelp->keyword	= fread_string( is, "Reading help keyword" );
	if ( pHelp->keyword[0] == '$' )
       {
		 pHelp.do_delete();
	    break;
       }
	pHelp->text	= fread_string( is, "Reading help text" );

   TheHelps.Add(pHelp);
   strcpy(szLastSuccess, pHelp->keyword.c_str());
    }

    return;
}



/*
 * Snarf a recall point.
 */
void TAreaDocument::load_recall( istream& is )
{
    recall = (sh_int) fread_number( is, "Reading recall room number" );

    if ( recall < 1 )
//        bug( "Load_recall:  %s invalid recall point", NULL );
       SyntaxError(is, "Reading recall room number", "Invalid room number");
}

/* race table */
struct race_type FAR	race_table[] =
{
/*
    {
	name,		pc_race?,
	act bits,	aff_by bits,	off bits,
	imm,		res,		vuln,
	form,		parts
    },
*/
    { "unique",		FALSE, 0, 0, 0, 0, 0, 0, 0, 0 },

#if 0 // @@@ Commented out...  No races yet...
    {
	"human",		TRUE,
	0,		0, 		0,
	0, 		0,		0,
	A|H|M|V,	A|B|C|D|E|F|G|H|I|J|K
    },

    {
	"elf",			TRUE,
	0,		AFF_INFRARED,	0,
	0,		RES_CHARM,	VULN_IRON,
	A|H|M|V,	A|B|C|D|E|F|G|H|I|J|K
    },

    {
	"dwarf",		TRUE,
	0,		AFF_INFRARED,	0,
	0,		RES_MAGIC|RES_POISON|RES_DISEASE, VULN_DROWNING,
	A|H|M|V,	A|B|C|D|E|F|G|H|I|J|K
    },

    {
	"giant",		TRUE,
	0,		0,		0,
	0,		RES_FIRE|RES_COLD,	VULN_MENTAL|VULN_LIGHTNING,
	A|H|M|V,	A|B|C|D|E|F|G|H|I|J|K
    },

    {
	"bat",			FALSE,
	0,		AFF_FLYING|AFF_DARK_VISION,	OFF_DODGE|OFF_FAST,
	0,		0,		VULN_LIGHT,
	A|G|W,		A|C|D|E|F|H|J|K|P
    },

    {
	"bear",			FALSE,
	0,		0,		OFF_CRUSH|OFF_DISARM|OFF_BERSERK,
	0,		RES_BASH|RES_COLD,	0,
	A|G|V,		A|B|C|D|E|F|H|J|K|U|V
    },

    {
	"cat",			FALSE,
	0,		AFF_DARK_VISION,	OFF_FAST|OFF_DODGE,
	0,		0,		0,
	A|G|V,		A|C|D|E|F|H|J|K|Q|U|V
    },

    {
	"centipede",		FALSE,
	0,		AFF_DARK_VISION,	0,
	0,		RES_PIERCE|RES_COLD,	VULN_BASH
    },

    {
	"dog",			FALSE,
	0,		0,		OFF_FAST,
	0,		0,		0,
	A|G|V,		A|C|D|E|F|H|J|K|U|V
    },

    // @@@
    {
	"doll",			FALSE,
	0,		0,		0,
	IMM_MAGIC,	RES_BASH|RES_LIGHT,
	VULN_SLASH|VULN_FIRE|VULN_ACID|VULN_LIGHTNING|VULN_ENERGY,
	E|J|M|FORM_COLD_BLOOD,	A|B|C|G|H|K
    },

    {
	"fido",			FALSE,
	0,		0,		OFF_DODGE|ASSIST_RACE,
	0,		0,			VULN_MAGIC,
	B|G|V,		A|C|D|E|F|H|J|K|Q|V
    },

    {
	"fox",			FALSE,
	0,		AFF_DARK_VISION,	OFF_FAST|OFF_DODGE,
	0,		0,		0,
	A|G|V,		A|C|D|E|F|H|J|K|Q|V
    },

    {
	"goblin",		FALSE,
	0,		AFF_INFRARED,	0,
	0,		RES_DISEASE,	VULN_MAGIC,
	A|H|M|V,	A|B|C|D|E|F|G|H|I|J|K
    },

    {
	"hobgoblin",		FALSE,
	0,		AFF_INFRARED,	0,
	0,		RES_DISEASE|RES_POISON,	0
    },

    {
	"kobold",		FALSE,
	0,		AFF_INFRARED,	0,
	0,		RES_POISON,	VULN_MAGIC,
	A|B|H|M|V,	A|B|C|D|E|F|G|H|I|J|K|Q
    },

    // @@@
    {
	"lizard",		FALSE,
	0,		0,		0,
	0,		RES_POISON,	VULN_COLD,
	A|G|FORM_REPTILE|FORM_COLD_BLOOD,	A|C|D|E|F|H|K|Q|V
    },

    {
	"modron",		FALSE,
	0,		AFF_INFRARED,		ASSIST_RACE|ASSIST_ALIGN,
	IMM_CHARM|IMM_DISEASE|IMM_MENTAL|IMM_HOLY|IMM_NEGATIVE,
			RES_FIRE|RES_COLD|RES_ACID,	0,
	H,		A|B|C|G|H|J|K
    },

    {
	"orc",			FALSE,
	0,		AFF_INFRARED,	0,
	0,		RES_DISEASE,	VULN_LIGHT,
	A|H|M|V,	A|B|C|D|E|F|G|H|I|J|K
    },

    {
	"pig",			FALSE,
	0,		0,		0,
	0,		0,		0,
	A|G|V,	 	A|C|D|E|F|H|J|K
    },

    {
	"rabbit",		FALSE,
	0,		0,		OFF_DODGE|OFF_FAST,
	0,		0,		0,
	A|G|V,		A|C|D|E|F|H|J|K
    },

    {
	"school monster",	FALSE,
	ACT_NOALIGN,		0,		0,
	IMM_CHARM|IMM_SUMMON,	0,		VULN_MAGIC,
	A|M|V,		A|B|C|D|E|F|H|J|K|Q|U
    },

    // @@@
    {
	"snake",		FALSE,
	0,		0,		0,
	0,		RES_POISON,	VULN_COLD,
	A|G|R|FORM_REPTILE|FORM_SNAKE|FORM_COLD_BLOOD,	A|D|E|F|K|L|Q|V|PART_SCALES
    },

    {
	"song bird",		FALSE,
	0,		AFF_FLYING,		OFF_FAST|OFF_DODGE,
	0,		0,		0,
	A|G|W,		A|C|D|E|F|H|K|P
    },

    {
	"troll",		FALSE,
	0,		AFF_REGENERATION|AFF_INFRARED|AFF_DETECT_HIDDEN,
	OFF_BERSERK,
 	0,	RES_CHARM|RES_BASH,	VULN_FIRE|VULN_ACID,
	B|M|V,		A|B|C|D|E|F|G|H|I|J|K|U|V
    },

    {
	"water fowl",		FALSE,
	0,		AFF_SWIM|AFF_FLYING,	0,
	0,		RES_DROWNING,		0,
	A|G|W,		A|C|D|E|F|H|K|P
    },

    {
	"wolf",			FALSE,
	0,		AFF_DARK_VISION,	OFF_FAST|OFF_DODGE,
	0,		0,		0,
	A|G|V,		A|C|D|E|F|J|K|Q|V
    },

    // @@@
    {
	"wyvern",		FALSE,
	0,		AFF_FLYING|AFF_DETECT_INVIS|AFF_DETECT_HIDDEN,
	OFF_BASH|OFF_FAST|OFF_DODGE,
	IMM_POISON,	0,	VULN_LIGHT,
	B|Z|FORM_COLD_BLOOD,		A|C|D|E|F|H|J|K|Q|V|PART_SCALES
    },
#endif

    {
	NULL, 0, 0, 0, 0, 0, 0
    }
};



/*
 * Snarf a mob section.
 */
void TAreaDocument::load_mobiles( istream& is, poundsApp::currMode mode )
{
//    PMobile pMobIndex;
    TMobile *pMobIndex;
    char szBuffer[256];

    for ( ; ; )
    {
	char letter;
	sh_int  vnum;

	letter				= fread_letter( is );
	if ( letter != '#' )
	{
       SyntaxError(is, "Looking for a mobile", "Did not find leading # symbol");
	}

	vnum				= (sh_int) fread_number( is, "Reading mobile vnum" );
	if ( vnum == 0 )
	    break;

	if ( get_mob_index( vnum ) )
	{
      wsprintf(szBuffer, "Two mobiles both using VNUM #%d", vnum);
      SyntaxError(is, "Loading mobile", szBuffer);
	}

   pMobIndex = new TMobile(this);
   TRY {
	pMobIndex->vnum		  = vnum;
	pMobIndex->player_name = fread_string( is, "Reading mobile name" );
	pMobIndex->short_descr = fread_string( is, "Reading mobile short descr" );
	pMobIndex->long_descr  = fread_string( is, "Reading mobile long descr" );
	pMobIndex->description = fread_string( is, "Reading mobile LOOK descr" );
  // if (mode == poundsApp::RomMode)
  // 	pMobIndex->race = race_lookup(fread_string( is, "Reading mobile race" ));


   if (!pMobIndex->long_descr.is_null())
      {
   	pMobIndex->long_descr[0] = UPPER( pMobIndex->long_descr[0]  );

      // Strip trailing newline;
      int n = pMobIndex->long_descr.length();
      while (n && pMobIndex->long_descr[n-1] < ' ')
         n--;
      pMobIndex->long_descr.resize(n);
      }

   if (!pMobIndex->description.is_null())
   	pMobIndex->description[0]	= UPPER( pMobIndex->description[0] );

//   if (mode == poundsApp::RomMode)
//      {
//      pMobIndex->act         = fread_flag( is, "Reading mobile ACT flags" ) |
//         race_table[pMobIndex->race].act;
//      pMobIndex->affected_by = fread_flag( is, "Reading mobile AFF flags" ) |
//         race_table[pMobIndex->race].aff;
//      }
//   else
//      {
      pMobIndex->act			  = fread_number( is, "Reading mobile ACT_xxx bit flags" ) /*| ACT_IS_NPC*/;
	   pMobIndex->affected_by = fread_number( is, "Reading mobile AFF_xxx bit flags" );
 //     }
	pMobIndex->pShop		  = NULL;
	pMobIndex->alignment	  = (sh_int) fread_number( is, "Reading mobile alignment" );
	letter				     = fread_letter( is );
	pMobIndex->level		  = (sh_int) fread_number( is, "Reading mobile level" );

	if ( letter != 'S' )
   	{
      wsprintf(szBuffer, "Expected S in mobile vnum #%d structure, got %c",
         vnum, letter);
      SyntaxError(is, "Loading mobile", szBuffer);
	   }

	/*
	 * The unused stuff is for imps who want to use the old-style
	 * stats-in-files method.
	 */
	pMobIndex->hitroll     = fread_number( is, "Skipping mobile hit roll" );   /* Unused */
   if (mode != poundsApp::RomMode)
   	pMobIndex->ac          = fread_number( is, "Skipping mobile armor class" );   /* Unused */
	pMobIndex->hitnodice   = fread_number( is, "Skipping mobile hit dice" );   /* Unused */
	/* 'd'		*/           fread_letter( is );   /* Unused */
	pMobIndex->hitsizedice = fread_number( is, "Skipping mobile hit dice size" );   /* Unused */
	/* '+'		*/           fread_letter( is );   /* Unused */
	pMobIndex->hitplus	  = fread_number( is, "Skipping mobile hit point bonus" );   /* Unused */
   if (mode == poundsApp::RomMode)
      {
 	   /* read mana dice */
	   pMobIndex->mana[DICE_NUMBER]	= (sh_int) fread_number( is, "Reading mobile mana dice" );
		/* d */	fread_letter( is );
	   pMobIndex->mana[DICE_TYPE]	= (sh_int) fread_number( is, "Reading mobile mana dice size" );
		/* + */	fread_letter( is );
	   pMobIndex->mana[DICE_BONUS]	= (sh_int) fread_number( is, "Reading mobile mana bonus" );
      }

	pMobIndex->damnodice   = fread_number( is, "Skipping mobile damage dice" );   /* Unused */
	/* 'd'		*/           fread_letter( is );   /* Unused */
	pMobIndex->damsizedice = fread_number( is, "Skipping mobile damage dice size" );   /* Unused */
	/* '+'		*/           fread_letter( is );   /* Unused */
	pMobIndex->damplus     = fread_number( is, "Skipping mobile damage bonus" );   /* Unused */
   if (mode == poundsApp::RomMode)
      {
	   pMobIndex->dam_type		= (sh_int) fread_number( is, "Reading mobile damage type" );

	   /* read armor class */
	   pMobIndex->rom_ac[AC_PIERCE]	= (sh_int) (fread_number( is, "Reading mobile AC vs pierce" ) * 10);
	   pMobIndex->rom_ac[AC_BASH]		= (sh_int) (fread_number( is, "Reading mobile AC vs bash" ) * 10);
	   pMobIndex->rom_ac[AC_SLASH]	= (sh_int) (fread_number( is, "Reading mobile AC vs slash" ) * 10);
	   pMobIndex->rom_ac[AC_EXOTIC]	= (sh_int) (fread_number( is, "Reading mobile AC vs exotic" ) * 10);

	   /* read flags and add in data from the race table */
	//   pMobIndex->off_flags		= fread_flag( is, "Reading mobile off flags" )
	// 				| race_table[pMobIndex->race].off;
//	   pMobIndex->imm_flags		= fread_flag( is, "Reading mobile imm flags" )
 //					| race_table[pMobIndex->race].imm;
//	   pMobIndex->res_flags		= fread_flag( is, "Reading mobile res flags" )
 //					| race_table[pMobIndex->race].res;
 //	   pMobIndex->vuln_flags		= fread_flag( is, "Reading mobile vuln flags" )
 //					| race_table[pMobIndex->race].vuln;

        pMobIndex->off_flags     = 0;
        fread_flag( is, "Reading mobile off flags" );
        pMobIndex->imm_flags 		= 0;
        fread_flag( is, "Reading mobile imm flags" );
        pMobIndex->res_flags 		= 0;
        fread_flag( is, "Reading mobile res flags" );
        pMobIndex->vuln_flags		= 0;
        fread_flag( is, "Reading mobile vuln flags" );

	   /* vital statistics */
	   pMobIndex->start_pos	= (sh_int) fread_number( is, "Reading mobile start pos" );
	   pMobIndex->default_pos = (sh_int) fread_number( is, "Reading mobile default pos" );
	   pMobIndex->sex			= (sh_int) fread_number( is, "Reading mobile sex" );
	   pMobIndex->gold		= fread_number( is, "Reading mobile gold" );

	//   pMobIndex->form		= fread_flag( is, "Reading mobile form" )
	//				| race_table[pMobIndex->race].form;
	//   pMobIndex->parts		= fread_flag( is, "Reading mobile parts" )
	//				| race_table[pMobIndex->race].parts;

        pMobIndex->form		= 0;
        fread_flag(is, "Reading mobile form" );
        pMobIndex->parts	= 0;
        fread_flag(is, "Reading mobile parts" );

	   /* size */
	   pMobIndex->size = fread_letter( is /*, "Reading mobile size"*/ );
    	//pMobIndex->material = material_lookup(fread_word( is, "Reading mobile material" ));
    	pMobIndex->material = fread_word( is, "Reading mobile material" );
      }
   else
      {
   	pMobIndex->gold = fread_number( is, "Skipping mobile gold amount" );   /* Unused */
	   /* xp        */   fread_number( is, "Skipping mobile XP" );   /* Unused */
	   /* position	 */   fread_number( is, "Skipping mobile position" );   /* Unused */
	   /* start pos */   fread_number( is, "Skipping mobile start position" );   /* Unused */
	   pMobIndex->sex	 = (sh_int) fread_number( is, "Reading mobile sex" );
      }


   for ( ; ; )
     {

        letter = fread_letter(is);

        if ( letter == 'C' || letter == 'c' )
        	{
           pMobIndex->clss = (sh_int) fread_number(is, "" );
         }

        else if ( letter == 'R' || letter == 'r' )
         {
           pMobIndex->race = fread_number(is, "" );
         }
        else if ( letter == '#' )
         {
          is.putback(letter);
          break;
			}
     }


 /*
    letter = fread_letter(is);
    is.putback(letter);

    if (letter == 'C')
       {
       	pMobIndex->clss = fread_number(is, "" );
      	fread_to_eol(is);
       }
    else if (letter == 'R')
       {
         pMobIndex->race = fread_number(is, "" );
      	fread_to_eol(is);
       }
*/

  //	letter=fread_letter(is);
//   is.putback(letter);
//	if (letter=='>')
 //	    mprog_read_programs(is,pMobIndex);



    if (nNextMobVNum < vnum)
       {
       // First mob seen?
       if (nNextMobVNum == -1)
          {
          if (nNextObjVNum == -1)
             nNextObjVNum = vnum;
          if (nNextRoomVNum == -1)
             nNextRoomVNum = vnum;
          }

       nNextMobVNum = vnum;
       }
    }
    CATCH( (xmsg& x) {
       // This mobile was invalid.  Clean up the work we did on it and rethrow.
       delete pMobIndex;
       throw;
    } )

    TheMobiles.Add(pMobIndex);
    strcpy(szLastSuccess, pMobIndex->short_descr.c_str());
    }
}


/* This routine reads in scripts of MOBprograms from a file */
#pragma argsused
PMobProgData TAreaDocument::mprog_file_read(const string& f, PMobProgData mprg,
   PMobile pMobIndex)
{
#if 1
//  MPROG_DATA* mprg2;
   PMobProgData mprg2;
//  FILE * progfile;
   char MOBProgfile[MAX_INPUT_LENGTH];
   char szMessage[256];
   int letter;

   fFoundMOBprog = TRUE;

//   wsprintf( MOBProgfile, "%s%s", MOB_DIR, f.c_str() );
   wsprintf( MOBProgfile, "%s%s", "", f.c_str() );
   ifstream progfile( MOBProgfile, ios::in | ios::nocreate | ios::binary );
   if (progfile.bad())
      {
      wsprintf(szMessage, "Couldn't open MOBprogram for Mobile #%d", pMobIndex->vnum);
      SyntaxError(progfile, "Opening MOBprogram file", szMessage);
//      bug("Mob:%d couldn't open mobprog file",pMobIndex->vnum);
      }

//   int letter;
   BOOL done=FALSE;
   mprg2=mprg;
   mprg2->MobFile = f;   // Filename of MOBprog
   switch (letter=fread_letter(progfile)) {
   case '>':
     break;
   case '|':
//       bug("empty mobprog file.", 0);
       SyntaxError(progfile, "Reading MOBprogram", "Empty MOBprogram file");
//       exit( 1 );
     break;
   default:
//       bug("in mobprog file syntax error.", 0);
       wsprintf(szMessage, "Expected > or | in MOBprogram, got %c", letter);
       SyntaxError(progfile, "Reading MOBprogram", szMessage);
//       exit( 1 );
   break;
   }

   while (!done)
      {
      mprg2->type = fread_word(progfile, "Reading MOBprogram trigger type");
      if (mprg2->type == "in_file_prog")
//         bug( "mprog file contains a call to file.", 0 );
         SyntaxError(progfile, "Loading MOBprogram file", "MOBprogram file contains another in_file_prog");
      else
         {
//         pMobIndex->progtypes = pMobIndex->progtypes | mprg2->type;
         mprg2->arglist=fread_string(progfile, "Reading MOBprogram trigger argument" );
         mprg2->comlist=fread_string(progfile, "Reading MOBprogram command list" );
         switch (letter=fread_letter(progfile)) {
         case '>':
            pMobIndex->mobprogs.Add(mprg2);
            mprg2 = new TMobProgData(this);
            mprg2->MobFile = f;   // Filename of MOBprog
            break;
         case '|':
             done=TRUE;
           break;
         default:
             wsprintf(szMessage, "Expecting > or |, got '%c'", letter);
             SyntaxError(progfile, "Reading MOBprograms", szMessage);
//           bug( "syntax error in mobprog.",0);
//           break;
         }
         }
      }

  return mprg2;
#else
   THROW(TXOwl("Sorry, Can't read MOBprograms yet"));
#endif
}

/* This procedure is responsible for reading any in_file MOBprograms.
 */
#pragma argsused
void TAreaDocument::mprog_read_programs(istream& is, PMobile pMobIndex)
{
//   MPROG_DATA * mprg;
   BOOL done=FALSE;
   char szBuffer[256];
   int letter;
   if ( (/*letter=*/fread_letter(is)) != '>' )
  {
//      bug( "Load_mobiles: vnum %d MOBPROG char", pMobIndex->vnum );
      wsprintf(szBuffer, "Missing > in MOBprogram for mobile vnum %d", pMobIndex->vnum);
      SyntaxError(is, "Reading MOBprogram", szBuffer);
//      exit( 1 );
  }

  fFoundMOBprog = TRUE;
  TMobProgData *mprg = new TMobProgData(this);

//  mprg=pMobIndex->mobprogs;
  while (!done)
  {
//    mprg->type=mprog_name_to_type(fread_word(fp));
    mprg->type=fread_word(is, "Reading MOBprogram trigger type");
//    switch (mprg->type)
//    {
//     case ERROR_PROG:
//        bug( "Load_mobiles: vnum %d MOBPROG type.", pMobIndex->vnum );
//        exit( 1 );
//      break;
//     case IN_FILE_PROG:
    if (mprg->type == "in_file_prog")
       {
        mprg=mprog_file_read(fread_string(is, "Reading MOBprogram filename"), mprg, pMobIndex);
        fread_to_eol(is);
        switch (letter=fread_letter(is))
        {
          case '>':
//             mprg->next=(MPROG_DATA *)alloc_perm(sizeof(MPROG_DATA));
//             mprg=mprg->next;
//             mprg->next=NULL;
           pMobIndex->mobprogs.Add(mprg);
           mprg = new TMobProgData(this);
           break;
          case '|':
//             mprg->next=NULL;
				 delete mprg;
             fread_to_eol(is);
             done=TRUE;
           break;
          default:
//             bug( "Load_mobiles: vnum %d bad MOBPROG.", pMobIndex->vnum );
             wsprintf(szBuffer, "Mobile vnum #%d expected > or | in MOBprogram, got %c",
                pMobIndex->vnum, letter);
             SyntaxError(is, "Reading MOBprogram", szBuffer);
//             exit( 1 );
           break;
        }
       }
//      break;
//     default:
     else
        {
//        pMobIndex->progtypes = pMobIndex->progtypes | mprg->type;
#if 1
        TArrayAsVector<TMobProgData*>* mobproglist = &pMobIndex->mobprogs;
#endif
        mprg->arglist=fread_string(is, "Reading MOBprogram trigger argument");
        fread_to_eol(is);
        mprg->comlist=fread_string(is, "Reading MOBprogram commands");
        fread_to_eol(is);
        switch (letter=fread_letter(is))
        {
          case '>':
           if (!mobproglist->Add(mprg))
//              bug("MProg_Read_Programs: Couldn't grow mobprog list", 0);
              SyntaxError(is, "Adding a new MOBprogram trigger", "Couldn't add trigger");
           mprg = new TMobProgData(this);
           break;
          case '|':
           if (!mobproglist->Add(mprg))
//              bug("MProg_Read_Programs: Couldn't grow mobprog list", 0);
              SyntaxError(is, "Adding a new MOBprogram trigger", "Couldn't add trigger");
             fread_to_eol(is);
             done=TRUE;
           break;
          default:
//             bug( "Load_mobiles: vnum %d bad MOBPROG.", pMobIndex->vnum );
             wsprintf(szBuffer, "Mobile vnum #%d expected > or |, got '%c'",
                letter);
             SyntaxError(is, "Reading MOBprogram", szBuffer);
//             exit( 1 );
           break;
        }
//      break;
       }
    }
//  }
}

/* Snarf a MOBprogram section from the area file.
 */
void TAreaDocument::load_mobprogs(istream &is)
{
   char letter;
   PMobile iMob;
   sh_int value;
   PMobProgData original, working;
   char szBuffer[256];

for (;;) switch (letter=fread_letter( is))
  {
  default:
     wsprintf(szBuffer, "Expected one of these characters: S, M, or *.  Got %c", letter);
     SyntaxError(is, "Reading a MOBprogram file name", szBuffer);
     break;
  case 'S':
  case 's':
     fread_to_eol(is);
     return;
  case '*':
     fread_to_eol(is);
     break;
  case 'M':
  case 'm':
     value = (sh_int) fread_number(is, "Reading a VNUM to attach MOBprogram");
     if ((iMob=get_mob_index(value))==NULL)
        {
        wsprintf(szBuffer, "MOBprogram for unknown mobile vnum #%d", value);
        SyntaxError(is, "Loading MOBprograms", szBuffer);
        }

     working = new TMobProgData(this);
     working=mprog_file_read(fread_word(is, "Reading MOBprogram from file"),working,iMob);
     iMob->mobprogs.Add(working);

     fread_to_eol(is);
     break;
   }
}

/*
 * Snarf an obj section.
 */
void TAreaDocument::load_objects( istream& is, poundsApp::currMode mode )
{
//    OBJ_INDEX_DATA *pObjIndex;
    PObject pObjIndex;
    char szBuffer[256];

    for ( ; ; )
	 {
	char  letter;
	sh_int   vnum;

	letter				= fread_letter( is );
	if ( letter != '#' )
       SyntaxError(is, "Looking for a new object", "Found something other than #");

	vnum				= (sh_int) fread_number( is, "Reading object VNUM" );
	if ( vnum == 0 )
	    break;

	if ( get_obj_index( vnum ) )
	{
      wsprintf(szBuffer, "Two objects trying to use vnum #%d", vnum);
      SyntaxError(is, "Reading objects", szBuffer);
	}

   pObjIndex = new TObject(this);
	pObjIndex->vnum			= vnum;
	pObjIndex->name			= fread_string( is, "Reading object name" );
	pObjIndex->short_descr	= fread_string( is, "Reading object short description" );
	pObjIndex->description	= fread_string( is, "Reading object description" );
   if (mode != poundsApp::RomMode)
	/* Action description */ fread_string( is, "Ignoring object action description" );
   else
		pObjIndex->material = fread_string( is, "Reading object material" );
//      pObjIndex->material = material_lookup(fread_string( fp, "Reading object material ));

   if (!pObjIndex->short_descr.is_null())
   	pObjIndex->short_descr[0]	= LOWER( pObjIndex->short_descr[0] );
   if (!pObjIndex->description.is_null())
   	pObjIndex->description[0]	= UPPER( pObjIndex->description[0] );

 	pObjIndex->item_type		= (sh_int) fread_number( is, "Reading object item type" );
   if (mode == poundsApp::RomMode)
      {
      pObjIndex->extra_flags          = fread_flag( is, "Reading object extra flags" );
      pObjIndex->wear_flags           = fread_flag( is, "Reading object wear position bits" );
      }
   else
      {
	   pObjIndex->extra_flags		= fread_number( is, "Reading object extra flags" );
	   pObjIndex->wear_flags		= fread_number( is, "Reading object wear position bits" );
      }


   if (mode == poundsApp::EnvyMode)
      {
	pObjIndex->value[0] = fread_string( is, "Reading object value0" );
	pObjIndex->value[1] = fread_string( is, "Reading object value1" );
	pObjIndex->value[2] = fread_string( is, "Reading object value2" );
	pObjIndex->value[3] = fread_string( is, "Reading object value3" );
   for (int i = 0; i < nelems(pObjIndex->value); i++)
      if (pObjIndex->value[i].length() > 32)
         SyntaxError(is, "Reading object values", "Value to long--Check file format");
      }
   if (mode == poundsApp::RomMode)
      {
      int32 n;
      char szBuffer[32];
      n = fread_flag( is, "Reading object value0" );
      wsprintf(szBuffer, "%ld", n);
	   pObjIndex->value[0] = szBuffer;
      n = fread_flag( is, "Reading object value1" );
      wsprintf(szBuffer, "%ld", n);
	   pObjIndex->value[1] = szBuffer;
      n = fread_flag( is, "Reading object value2" );
      wsprintf(szBuffer, "%ld", n);
	   pObjIndex->value[2] = szBuffer;
      n = fread_flag( is, "Reading object value3" );
      wsprintf(szBuffer, "%ld", n);
	   pObjIndex->value[3] = szBuffer;
	   n = fread_flag( is, "Reading object value4" );
      wsprintf(szBuffer, "%ld", n);
	   pObjIndex->value[4] = szBuffer;
      }
   else
		{
      int32 n;
      char szBuffer[32];
	n = fread_number( is, "Reading object value0--Check file format" );
   wsprintf(szBuffer, "%ld", n);
	pObjIndex->value[0]		= szBuffer;
	n = fread_number( is, "Reading object value1--Check file format" );
   wsprintf(szBuffer, "%ld", n);
	pObjIndex->value[1]		= szBuffer;
	n = fread_number( is, "Reading object value2--Check file format" );
   wsprintf(szBuffer, "%ld", n);
	pObjIndex->value[2]		= szBuffer;
	n = fread_number( is, "Reading object value3--Check file format" );
   wsprintf(szBuffer, "%ld", n);
	pObjIndex->value[3]		= szBuffer;
   pObjIndex->value[4] = "0";
      }

   if (mode == poundsApp::RomMode)
   	pObjIndex->level		= (sh_int) fread_number( is, "Reading object level" );

	pObjIndex->weight		= fread_number( is, "Reading object weight" );
	pObjIndex->cost			= fread_number( is, "Ignoring object cost" );
   if (mode == poundsApp::RomMode)
      {
      /* condition */
		pObjIndex->condition = fread_letter( is /*, "Reading object condition"*/ );
      }
   else
   	/* Cost per day */ fread_number( is, "Ignoring object rent" );   /* Unused */

	/*
	 * Check here for the redundancy of invisible light sources - Kahn
	 */
	if ( pObjIndex->item_type == ITEM_LIGHT
	    && IS_SET( pObjIndex->extra_flags, ITEM_INVIS ) )
	{
       // @@@ Add this to the warnings list when ready
       char szBuffer[128];
       wsprintf(szBuffer,
          "Object #%d: light source is invisible.  Make visible?", vnum );
       if (MessageBox(0, szBuffer, "MAKE.ZONES.FAST", MB_ICONQUESTION|MB_YESNO)
             == IDYES)
   	    REMOVE_BIT( pObjIndex->extra_flags, ITEM_INVIS );
	}

	for ( ; ; )
	{
	    char letter;

	    letter = fread_letter( is );

		 if ( letter == 'A' )
	       {
          PAffect paf = new TAffect;
   		 paf->location		= (sh_int) fread_number( is, "Reading object apply location" );
	   	 paf->modifier		= (sh_int) fread_number( is, "Reading object apply amount" );
          pObjIndex->affected.Add(paf);
	       }

	    else if ( letter == 'E' )
	       {
          PExtraDescr ed = new TExtraDescr();

		    ed->keyword		= fread_string( is, "Reading object extra description keyword" );
  		    ed->description	= fread_string( is, "Reading object extra description" );
          pObjIndex->extra_descr.Add(ed);
	       }

       else if ( letter == 'Q' )
       	 {
          	/* What the hell is this?! */ 	 fread_number(is, "Reading Q flag" );
          }

	    else
	       {
          is.putback(letter);
		    break;
	       }
	}

   if (mode == poundsApp::EnvyMode)
      {
		char szBuffer[32];

	/*
	 * Translate character strings *value[] into integers:
	 */
	switch ( pObjIndex->item_type )
	{
	case ITEM_PILL:
	case ITEM_POTION:
	case ITEM_SCROLL:
       wsprintf(szBuffer, "%d", SpellValue( pObjIndex->value[1].c_str()));
	    pObjIndex->value[1] = szBuffer;
       wsprintf(szBuffer, "%d", SpellValue( pObjIndex->value[2].c_str()));
	    pObjIndex->value[2] = szBuffer;
       wsprintf(szBuffer, "%d", SpellValue( pObjIndex->value[3].c_str()));
	    pObjIndex->value[3] = szBuffer;
	    break;

	case ITEM_STAFF:
	case ITEM_WAND:
       wsprintf(szBuffer, "%d", SpellValue( pObjIndex->value[3].c_str()));
	    pObjIndex->value[3] = szBuffer;
	    break;
	}
   }

	if (nNextObjVNum < vnum)
       {
       // First obj seen?
       if (nNextObjVNum == -1)
          {
          if (nNextRoomVNum == -1)
             nNextRoomVNum = vnum;
          if (nNextMobVNum == -1)
             nNextMobVNum = vnum;
          }

       nNextObjVNum = vnum;
		 }

	 TheObjects.Add(pObjIndex);
	 strcpy(szLastSuccess, pObjIndex->short_descr.c_str());
	 }
}



