#include "areadoc.h"

#include "roomdia.h"
#include "objdia.h"
#include "mobdia.h"
#include "msetdia.h"
#include "shopdia.h"
#include "helpdia.h"

poundsApp *TheMuditorApp;

/*
 * Removes the tildes from a string.
 * Used for player-entered strings that go into disk files.
 */
void TAreaDocument::smash_tilde( string& str )
{
    for (int n = str.length()-1; n >= 0; n--)
       if (str[n] == '~')
          str[n] = '-';
}

/*
 * Removes the tildes from a string.
 * Used for player-entered strings that go into disk files.
 */
void TAreaDocument::trim( string& str )
{
    for (int n = str.length()-1; n >= 0; n--)
       if (str[n] > ' ')
          break;
       else
          str.resize(n);
}

#if 0
TVNumObj *TAreaDocument::GetVNumIndex(int nWhich, sh_int vnum)
{
   TArrayAsVector<TVNumObj *> *p;
   switch (nWhich)
      {
      case 0:
         p = &TheMobiles;
         break;
      case 1:
         p = &TheObjects;
         break;
      case 2:
         p = &TheRooms;
         break;
      default:
         bug("GetVNumIndex(): Bad nWhich");
      }

   for (TArrayAsVectorIterator<TVNumObj *> it(*p); it; it++)
      if (it.Current()->vnum == vnum)
         return it.Current();

   return 0;
}
#endif

const TMercObj *TAreaDocument::GetItem(int nWhich, int nIndex, char *lpOut,
   int nOutSize)
{
//   TArrayAsVector<TVNumObj *> *p;
   switch (nWhich)
      {
      case 0:
         return (nIndex >= TheMobiles.GetItemsInContainer()) ? 0 :
                                   TheMobiles[nIndex]->GetName(lpOut, nOutSize);
      case 1:
         return (nIndex >= TheObjects.GetItemsInContainer()) ? 0 :
                                   TheObjects[nIndex]->GetName(lpOut, nOutSize);
      case 2:
         return (nIndex >= TheRooms.GetItemsInContainer()) ? 0 :
                                   TheRooms[nIndex]->GetName(lpOut, nOutSize);
      case 3:
         return (nIndex >= TheHelps.GetItemsInContainer()) ? 0 :
                                   TheHelps[nIndex]->GetName(lpOut, nOutSize);
      case 4:
         return (nIndex >= TheShops.GetItemsInContainer()) ? 0 :
                                   TheShops[nIndex]->GetName(lpOut, nOutSize);
      case 5:
         return (nIndex >= TheResets.GetItemsInContainer()) ? 0 :
                                   TheResets[nIndex]->GetName(lpOut, nOutSize);
      case 6:
         return (nIndex >= TheMobResets.GetItemsInContainer()) ? 0 :
                                   TheMobResets[nIndex]->GetName(lpOut, nOutSize);
      }

//   bug("GetVNumIndex(): Bad nWhich");
   THROW( TXOwl("GetVNumIndex(): Bad nWhich") );

   // NOTREACHED
//   return 0;
}

TMobile* TAreaDocument::get_mob_index	( sh_int vnum )
{
   if (!IsOpen() && !Open(ofRead|ofWrite))
      return 0;

   for (TArrayAsVectorIterator<TMobile *> it(TheMobiles); it; it++)
      if (it.Current()->vnum == vnum)
         return it.Current();

   return 0;
}

TObject* TAreaDocument::get_obj_index	( sh_int vnum )
{
   if (!IsOpen() && !Open(ofRead|ofWrite))
      return 0;

   for (TArrayAsVectorIterator<TObject *> it(TheObjects); it; it++)
      if (it.Current()->vnum == vnum)
         return it.Current();

   return 0;
}

TRoom* TAreaDocument::get_room_index	( sh_int vnum )
{
   if (!IsOpen() && !Open(ofRead|ofWrite))
      return 0;

   for (TArrayAsVectorIterator<TRoom *> it(TheRooms); it; it++)
      if (it.Current()->vnum == vnum)
         return it.Current();

   return 0;
}

void TAreaDocument::DeleteItem(int nWhich, int nIndex)
{
   ChangeNotice cn = { nWhich, nIndex };
   int nMobResetCount;

//   int nExitCount;

   // @@@ Note:  Certain views depend on data from other views.  For example,
   // Shops needs mobs, Resets needs mobs, objs and rooms, MobResets needs
   // mobs, objs, rooms and resets.
   switch (nWhich)
      {
      case 0: // Mobiles
         cn.pObj = TheMobiles[nIndex];
         // Check for mob resets, issue warning
         TheMobiles.Detach(nIndex);
         break;
      case 1: // Objects
         cn.pObj = TheObjects[nIndex];
         // @@@ Check for resets, issue warning
         TheObjects.Detach(nIndex);
         break;
      case 2: // Rooms
         cn.pObj = TheRooms[nIndex];

         {
         // Are any mobs reset in this room?
         nMobResetCount = 0;
         for (TArrayAsVectorIterator<TMobReset *> it2(TheMobResets); it2; it2++)
            // arg3 is the room index
            if (it2.Current()->arg3 == TheRooms[nIndex]->vnum)
               nMobResetCount++;
         }

         if (nMobResetCount)
            if (MessageBox(0, "Warning:  Editor will not automatically delete "
                  "mobiles reset to this room.  Continue?", "MAKE.ZONES.FAST",
                  MB_ICONQUESTION|MB_YESNO) == IDNO)
               return;

         {
         // Delete the exits other rooms have to this room.
//         nExitCount = 0;
         for (TArrayAsVectorIterator<TRoom *> it(TheRooms); it; it++)
            for (int i = 0; i < 6; i++)
               if (it.Current()->exit[i].vnum == TheRooms[nIndex]->vnum)
                  {
                  it.Current()->is_exit[i] = FALSE;
//                  nExitCount++;
                  }
         }

         TheRooms.Detach(nIndex);
         break;
      case 3: // Helps
         cn.pObj = TheHelps[nIndex];
         TheHelps.Detach(nIndex);
         break;
      case 4: // Shops
         cn.pObj = TheShops[nIndex];
         TheShops.Detach(nIndex);
         break;
      case 5: // Resets
#if 1
         MessageBox(0, "Sorry, Resets can't be deleted.", "MZF", MB_OK);
         return;
#else
         cn.pObj = TheResets[nIndex];
         TheResets.Detach(nIndex);

         // If the reset is duplicated in mob resets, get rid of it there too
         {
         for (TArrayAsVectorIterator<TMobReset *> it(TheMobResets); it; it++)
            if (it.Current() == cn.pObj)
               {
               TheMobResets.Detach(it.Current());
               break;
               }
         }
         break;
#endif
      case 6: // MobResets
#if 0
         MessageBox(0, "Sorry, Mob Resets can't be deleted.", "MZF", MB_OK);
         return;
#else
         {
         // Look up this reset
         PMobReset pMobReset = TheMobResets[nIndex];

         // Get rid of the mobile's wielded eq
         ChangeNotice cn1 = { 5 };
         for (TArrayAsVectorIterator<TReset*> itWields(pMobReset->Wields); itWields; itWields++)
            {
            cn1.nIndex = GetIndexFromReset(itWields.Current());
            cn1.pObj = itWields.Current();
            TheResets.Detach(itWields.Current());
            NotifyViews(vnAreaDelete, (long) &cn1);
            }

         // Get rid of the mobile's inventory
         for (TArrayAsVectorIterator<TReset*> itInv(pMobReset->Inventory); itInv; itInv++)
            {
            cn1.nIndex = GetIndexFromReset(itInv.Current());
            cn1.pObj = itInv.Current();
            TheResets.Detach(itInv.Current());
            NotifyViews(vnAreaDelete, (long) &cn1);
            }

         // Remove the mob reset from the TRoom that keeps it...
         PRoom pRoom = get_room_index ( pMobReset->arg3 );
         if (pRoom)
            pRoom->MobResets.Detach(pMobReset);

         // Get rid of the mobile itself
         cn.pObj = pMobReset;
         TheMobResets.Detach(nIndex);

         // If the mob reset is duplicated in resets, get rid of it there too
         cn1.nIndex = GetIndexFromReset(pMobReset);
         cn1.pObj = pMobReset;
         TheResets.Detach(pMobReset);
         NotifyViews(vnAreaDelete, (long) &cn1);
         }
         break;
#endif
      }

  SetDirty(TRUE);
  NotifyViews(vnAreaDelete, (long) &cn);
}

void TAreaDocument::EditItem(int nWhich, int nIndex)
{
   TObject *obj;
   TMobile *mob;
   TRoom *room;
   TMobReset *mobreset;
   TShopData *shop;
   THelpData *help;

   // This prevents an app fault if a new document is created and then a view
   // to something without vnums is opened and 'add' is selected
   if (!IsOpen() && !Open(ofRead|ofWrite))
      return;

   switch (nWhich)
      {
      case 0:
         mob = (nIndex < 0) ? MakeNewMobile() : TheMobiles[nIndex];
         TMobDialog(GetDocManager().GetApplication()->GetMainWindow(),
            mob, nIndex).Execute();
         if (nIndex < 0)
            delete mob;
         break;
      case 1:
         obj = (nIndex < 0) ? MakeNewObject() : TheObjects[nIndex];
         TObjDialog(GetDocManager().GetApplication()->GetMainWindow(),
            obj, nIndex).Execute();
         if (nIndex < 0)
            delete obj;
         break;
      case 2:
         room = (nIndex < 0) ? MakeNewRoom() : TheRooms[nIndex];
         TRoomDialog(GetDocManager().GetApplication()->GetMainWindow(),
            room, nIndex).Execute();
//         if (nIndex < 0)
//            delete room;
         break;
      case 3:
         help = (nIndex < 0) ? MakeNewHelp() : TheHelps[nIndex];
         THelpDialog (GetDocManager().GetApplication()->GetMainWindow(), help,
            nIndex).Execute();
         if (nIndex < 0)
            delete help;
         break;
      case 4:
         shop = (nIndex < 0) ? MakeNewShop() : TheShops[nIndex];
//         if (nIndex < 0)
//            MessageBox(0, "Sorry, you can't make new shops yet...", "MZF", MB_OK);
//         else
            TShopDialog(GetDocManager().GetApplication()->GetMainWindow(),
               this, shop, nIndex, FALSE).Execute();
         if (nIndex < 0)
            delete shop;
         break;
      case 5:
         MessageBox(0, "Sorry, Reset editing is not ready.", "MZF", MB_OK);
         break;
      case 6:
         mobreset = (nIndex < 0) ? MakeNewMobReset() : TheMobResets[nIndex];
         TMobResetDialog(GetDocManager().GetApplication()->GetMainWindow(),
            mobreset, TRUE, nIndex).Execute();
         if (nIndex < 0)
            delete mobreset;
         break;
      }
}

void TAreaDocument::CopyItem(int nWhich, int nIndex)
{
   if (nIndex < 0)
      return;

//   TObject *obj;
//   TMobile *mob;
//   TRoom *room;
//   TShopData *shop;

   HANDLE h = GlobalAlloc(GMEM_MOVEABLE, 32767);
   ostrstream buf((char *) GlobalLock(h), 32767, ios::out|ios::binary);
   TClipboard& clip = GetDocManager().GetApplication()->GetMainWindow()->OpenClipboard();
//   TClipboard clip;

   switch (nWhich)
      {
      case 0:
         WriteMobile(TheMobiles[nIndex], buf);
         break;
      case 1:
         WriteObject(TheObjects[nIndex], buf);
         break;
      case 2:
         WriteRoom(TheRooms[nIndex], buf);
         break;
      case 3:
         goto error; // @@@
      case 4:
         WriteShop(TheShops[nIndex], buf);
         break;
      case 5:
         WriteReset(TheResets[nIndex], buf);
         break;
      case 6:
         WriteMobReset(TheMobResets[nIndex], buf);
         break;
      }

//   clip.OpenClipboard(GetDocManager().GetApplication()->GetMainWindow()->HWindow);
   buf << '\0';
   if (clip.EmptyClipboard())
      clip.SetClipboardData(CF_TEXT, h);
   clip.CloseClipboard();
   return;

error:
   GlobalUnlock(h);
   GlobalFree(h);
}

const TMobile* TAreaDocument::GetMobileFromIndex(int nNdx) const
{
   return (nNdx >= TheMobiles.GetItemsInContainer()) ? 0 : TheMobiles[nNdx];
}

void TAreaDocument::MobileSave(const TMobile& mob, int nIndex)
{
   ChangeNotice cn = { 0, nIndex };

   if (nIndex < 0)
      {
      SetDirty(TRUE);
      cn.nIndex = TheMobiles.GetItemsInContainer();
      TheMobiles.Add(new TMobile(mob));
      NotifyViews(vnAreaAppend, (long) &cn);
    	nDefaultAction = mob.act;
	   nDefaultAffectedBy = mob.affected_by;
      }
   else
      {
      if (!(*TheMobiles[nIndex] == mob))
         SetDirty(TRUE);
      *TheMobiles[nIndex] = mob;
      NotifyViews(vnAreaModify, (long) &cn);
      }
}

TMobile* TAreaDocument::MakeNewMobile()
{
   TMobile *mob = new TMobile(this);

	mob->player_name = "mob";
	mob->short_descr = "the mob";
	mob->long_descr = "the mob stands here, completely clueless.";
	mob->description = "the mob looks back at you!\r\n";
	mob->vnum = NextMobVnum();
	mob->sex = SEX_MALE;
	mob->level = 1;
	mob->act = nDefaultAction;
	mob->affected_by = nDefaultAffectedBy;
	mob->alignment = 0;

   return mob;
}

sh_int TAreaDocument::NextMobVnum()
{
   while (GetMobile(nNextMobVNum))
      nNextMobVNum++;

   return nNextMobVNum;
}

const TRoom* TAreaDocument::GetRoomFromIndex(int nNdx) const
{
   return (nNdx >= TheRooms.GetItemsInContainer()) ? 0 : TheRooms[nNdx];
}

int TAreaDocument::GetIndexFromRoom(const TRoom *r) const
{
   int nNdx = 0;
   for ( TArrayAsVectorIterator<TRoom *> it(TheRooms); it; it++, nNdx++)
      if (it.Current() == r)
         return nNdx;

   // If the room is not in the structure, return must be -1 for a created room.
//   // @@@Note cast away const...  Make this a static...
//   ((TAreaDocument*) this)->bug("TAreaDocument::GetIndexFromRoom(): Couldn't find that room...");

//   // NOTREACHED
   return -1;
}

// @@@ Make part of class...
static TRoom *BeingAdded;

TRoom *TAreaDocument::RoomSave(const TRoom& room, int nIndex)
{
   ChangeNotice cn = { 2, nIndex };

   if (nIndex < 0)
      {
      nDefaultSectorFlags = room.room_flags;
      nDefaultSectorType = room.sector_type;

      SetDirty(TRUE);
      cn.nIndex = TheRooms.GetItemsInContainer();
      *BeingAdded = room;
      TheRooms.Add(BeingAdded);
      NotifyViews(vnAreaAppend, (long) &cn);
      TRoom *r = BeingAdded;
      BeingAdded = 0;
      return r;
      }

   if (!(*TheRooms[nIndex] == room))
      SetDirty(TRUE);
   *TheRooms[nIndex] = room;
   NotifyViews(vnAreaModify, (long) &cn);
   return TheRooms[nIndex];
}

TRoom *TAreaDocument::MakeNewRoom()
{
   char szBuffer[256];

   TRoom *room = new TRoom(this);
   room->name = "A room";
   room->vnum = NextRoomVnum();
   wsprintf(szBuffer, "You are standing in room #%d.\r\n", room->vnum);
   room->description = szBuffer;
   room->room_flags = nDefaultSectorFlags;
   room->sector_type = nDefaultSectorType;

   BeingAdded = room;
   return room;
}

sh_int TAreaDocument::NextRoomVnum()
{
   while (GetRoom(nNextRoomVNum))
      nNextRoomVNum++;

   return nNextRoomVNum;
}

const TObject* TAreaDocument::GetObjectFromIndex(int nNdx) const
{
   return (nNdx >= TheObjects.GetItemsInContainer()) ? 0 : TheObjects[nNdx];
}

void TAreaDocument::ObjectSave(const TObject& obj, int nIndex)
{
   ChangeNotice cn = { 1, nIndex };

   if (nIndex < 0)
      {
      SetDirty(TRUE);
      cn.nIndex = TheObjects.GetItemsInContainer();
      TheObjects.Add(new TObject(obj));
      NotifyViews(vnAreaAppend, (long) &cn);
      }
   else
      {
      if (!(*TheObjects[nIndex] == obj))
         {
         SetDirty(TRUE);
         *TheObjects[nIndex] = obj;
         }
      NotifyViews(vnAreaModify, (long) &cn);
      }
}

TObject* TAreaDocument::MakeNewObject()
{
   TObject *obj = new TObject(this);

   obj->name = "sword";
   obj->short_descr = "a sword";
   obj->description = "A sword rests here.";
   obj->vnum = NextObjVnum();
   obj->item_type = ITEM_WEAPON;
   obj->extra_flags = 0;
   obj->wear_flags = ITEM_TAKE|ITEM_WIELD;
   obj->weight = 5;        // sh_int in Merc, int in Envy

   obj->value[0] = "0";
   obj->value[1] = "0";
   obj->value[2] = "0";
   obj->value[3] = "0";

   return obj;
}

sh_int TAreaDocument::NextObjVnum()
{
   while (GetObject(nNextObjVNum))
      nNextObjVNum++;

   return nNextObjVNum;
}

poundsApp::currMode TAreaDocument::GetMode() const
{
    return MuditorApp->CurrentMode;
}

void TAreaDocument::MobilesToCombo(TComboBoxData& box, sh_int vnum) const
{
   char szBuffer[1024];

   for ( TArrayAsVectorIterator<TMobile *> it(TheMobiles); it; it++)
      {
      it.Current()->GetName(szBuffer, sizeof szBuffer);
      box.AddStringItem(szBuffer, it.Current()->vnum,
         it.Current()->vnum  == vnum);
      }
}

void TAreaDocument::RoomsToCombo(TComboBoxData& box, sh_int vnum) const
{
   char szBuffer[1024];

   for ( TArrayAsVectorIterator<TRoom *> it(TheRooms); it; it++)
      {
      it.Current()->GetName(szBuffer, sizeof szBuffer);
      box.AddStringItem(szBuffer, it.Current()->vnum,
         it.Current()->vnum  == vnum);
      }
}

void TAreaDocument::RoomsToList(TListBoxData& box, sh_int nMobVnum) const
{
   char szBuffer[1024];

   BOOL fFlag = TRUE;
   for ( TArrayAsVectorIterator<TMobReset *> it(TheMobResets); it; it++)
      if (it.Current()->arg1 == nMobVnum)
         {
         // @@@ CAST AWAY CONST
         const TRoom *pRoom = ((TAreaDocument*) this)->GetRoom(it.Current()->arg3);
         if (pRoom)
            pRoom->GetName(szBuffer, sizeof szBuffer);
         else
            wsprintf(szBuffer, "#%d", it.Current()->arg3);

         box.AddStringItem(szBuffer, (DWORD) it.Current(), fFlag);
         fFlag = FALSE;
         }
}

// Flags are considered to be ORs, so WEAR_WIELD|WEAR_ABOUT_BODY gives sword&robe
void TAreaDocument::ObjectsToComboWear(TComboBoxData& box, int32 wear_pos,
   sh_int vnum)
{
   char szBuffer[1024];

   for ( TArrayAsVectorIterator<TObject *> it(TheObjects); it; it++)
      if (it.Current()->wear_flags & wear_pos)
         {
         it.Current()->GetName(szBuffer, sizeof szBuffer);
         box.AddStringItem(szBuffer, it.Current()->vnum,
            it.Current()->vnum  == vnum);
         }
}

void TAreaDocument::ObjectsToComboType(TComboBoxData& box, sh_int type,
   sh_int vnum) const
{
   char szBuffer[1024];

   for ( TArrayAsVectorIterator<TObject *> it(TheObjects); it; it++)
      if (it.Current()->item_type == type)
         {
         it.Current()->GetName(szBuffer, sizeof szBuffer);
         box.AddStringItem(szBuffer, it.Current()->vnum,
            it.Current()->vnum  == vnum);
         }
}

void TAreaDocument::ObjectsToList(TListBoxData& box) const
{
   char szBuffer[1024];
   for ( TArrayAsVectorIterator<TObject *> it(TheObjects); it; it++)
      {
      it.Current()->GetName(szBuffer, sizeof szBuffer);
      box.AddStringItem(szBuffer, it.Current()->vnum);
      }
}

THelpData *TAreaDocument::MakeNewHelp()
{
   return new THelpData(this);
}

void TAreaDocument::HelpSave(const THelpData& help, int nIndex)
{
   ChangeNotice cn = { 3, nIndex };

   if (nIndex < 0)
      {
      SetDirty(TRUE);
      cn.nIndex = TheHelps.GetItemsInContainer();
      TheHelps.Add(new THelpData(help));
      NotifyViews(vnAreaAppend, (long) &cn);
      }
   else
      {
      if (!(*TheHelps[nIndex] == help))
         {
         SetDirty(TRUE);
         *TheHelps[nIndex] = help;
         }
      NotifyViews(vnAreaModify, (long) &cn);
      }
}

TShopData *TAreaDocument::MakeNewShop()
{
   return new TShopData(this);
}

void TAreaDocument::ShopSave(const TShopData& shop, int nIndex)
{
   ChangeNotice cn = { 4, nIndex };

   if (nIndex < 0)
      {
      SetDirty(TRUE);
      cn.nIndex = TheShops.GetItemsInContainer();
      TheShops.Add(new TShopData(shop));
      NotifyViews(vnAreaAppend, (long) &cn);
      }
   else
      {
      if (!(*TheShops[nIndex] == shop))
         {
         SetDirty(TRUE);
         *TheShops[nIndex] = shop;
         }
      NotifyViews(vnAreaModify, (long) &cn);
      }
}

int TAreaDocument::GetShopIndexFromMobileVNum(sh_int mobile)
{
   if (!IsOpen() && !Open(ofRead|ofWrite))
      return -1;

   int nNdx = 0;
   for (TArrayAsVectorIterator<TShopData *> it(TheShops); it; it++, nNdx++)
      if (it.Current()->keeper == mobile)
         return nNdx;

   return -1;
}

TMobReset *TAreaDocument::MakeNewMobReset()
{
   TMobReset *mobreset = new TMobReset(this);
//   mobreset->command = 'M';
   mobreset->arg2 = 1;
   return mobreset;
}

void TAreaDocument::SaveMobReset(const TMobReset& mset, int nIndex)
{
   ChangeNotice cn = { 6, nIndex };

   if (nIndex < 0)
      {
      SetDirty(TRUE);

      // First, add a leading empty comment line
      TReset *comment = new TReset(mset.AreaDoc);
      SaveReset(*comment, -1);
      delete comment;
      
      cn.nIndex = TheMobResets.GetItemsInContainer();
      TMobReset *newMSet = new TMobReset(mset);

      TheMobResets.Add(newMSet);
      TRoom *room = get_room_index(mset.arg3);
      if (room)
         room->MobResets.Add(newMSet);
      NotifyViews(vnAreaAppend, (long) &cn);

      // Notify the #RESETs view
#if 1
      cn.nWhich = 5;
      cn.nIndex = TheResets.GetItemsInContainer();
      TheResets.Add(newMSet);
      NotifyViews(vnAreaAppend, (long) &cn);
      for (TArrayAsVectorIterator<TReset *> it(newMSet->Wields); it; it++)
         {
         cn.nIndex = TheResets.GetItemsInContainer();
         TheResets.Add(it.Current());
         NotifyViews(vnAreaAppend, (long) &cn);
         }
      for (TArrayAsVectorIterator<TReset *> it2(newMSet->Inventory); it2; it2++)
         {
         cn.nIndex = TheResets.GetItemsInContainer();
         TheResets.Add(it2.Current());
         NotifyViews(vnAreaAppend, (long) &cn);
         }
#else
      // @@@ This will app fault when saving, also doesn't display wields
      SaveReset(mset, nIndex);
#endif
      }
   else
      {
      if (!(*TheMobResets[nIndex] == mset))
         {
         SetDirty(TRUE);
         *TheMobResets[nIndex] = mset;
         }

      NotifyViews(vnAreaModify, (long) &cn);

      // @@@ Add code to update the resets view...
      }
}

const TMobReset* TAreaDocument::GetMobResetFromIndex(int nNdx) const
{
   return (nNdx >= TheMobResets.GetItemsInContainer()) ? 0 : TheMobResets[nNdx];
}

int TAreaDocument::GetIndexFromMobReset(const TMobReset *mres) const
{
   int nNdx = 0;
   for ( TArrayAsVectorIterator<TMobReset *> it(TheMobResets); it; it++, nNdx++)
      if (it.Current() == mres)
         return nNdx;

   return -1;
}

TReset *TAreaDocument::SaveReset(const TReset& rset, int nIndex)
{
   ChangeNotice cn = { 5, nIndex };

   if (nIndex < 0)
      {
      SetDirty(TRUE);
      cn.nIndex = TheResets.GetItemsInContainer();
      TReset *newRSet = new TReset(rset);
      TheResets.Add(newRSet);
      NotifyViews(vnAreaAppend, (long) &cn);
      return newRSet;
      }
   else
      {
      if (!(*TheResets[nIndex] == rset))
         {
         SetDirty(TRUE);
         *TheResets[nIndex] = rset;
         }

      NotifyViews(vnAreaModify, (long) &cn);
      }

   return TheResets[nIndex];
}

int TAreaDocument::GetIndexFromReset(const TReset *res)
{
   int nNdx = 0;
   for ( TArrayAsVectorIterator<TReset *> it(TheResets); it; it++, nNdx++)
      if (it.Current() == res)
         return nNdx;

   return -1;
}

bool TAreaDocument::Commit (bool force)
{
	 if (!IsDirty() && !force)
		 return TRUE;

    // @@@ This should be ok from Open(), don't know...
    TheMuditorApp = MuditorApp =
       TYPESAFE_DOWNCAST(GetDocManager().GetApplication(), poundsApp);

    // @@@
    if (MuditorApp->CurrentMode == poundsApp::RomMode)
       {
       GetDocManager().GetApplication()->GetMainWindow()->MessageBox(
          "Saving in ROM mode is not ready.  ROM mode is currently only "
          "for conversion to other file formats.  Look for better ROM support "
          "in the next version", "M.Z.F.", MB_ICONHAND|MB_OK);
       return TRUE;
       }

	 TPointer<TOutStream> os = OutStream(ofWrite|ofBinary);
	 if (!os)
		 return FALSE;

//    if (!fUsesAreaData)
    	 *os << "#AREA\t" << name << "~\r\n\r\n\r\n";
//    else
       WriteAreaData(*os);

	 WriteHelps(*os);
//    if (!fUsesAreaData && recall != -1 &&
 //                           MuditorApp->CurrentMode == poundsApp::EnvyMode)
//       *os << "#RECALL\r\n" << recall << "\r\n";
	 WriteMobs(*os);
    WriteMobProgs(*os);
	 WriteObjects(*os);
	 WriteRooms(*os);
	 WriteResets(*os);
	 WriteShops(*os);
	 WriteSpecials(*os);
	 *os << "#$\r\n";

    BOOL status = os->good();
	 SetDirty(FALSE);
	 if (!status)
		PostError(IDS_WRITEERROR, MB_OK);
	 return status;
}

int TAreaDocument::exitinfo2locktype(int exitinfo)
{
   if ((exitinfo & EX_ISDOOR) != EX_ISDOOR)  // no door at all
      return 0;

   return (((exitinfo & EX_PICKPROOF) == EX_PICKPROOF) ? 1 : 0) +
          (((exitinfo & EX_BASHPROOF) == EX_BASHPROOF) ? 2 : 0) +
          (((exitinfo & EX_PASSPROOF) == EX_PASSPROOF) ? 4 : 0) + 1;
}

void TAreaDocument::WriteRooms(ostream& os)
{
	if (TheRooms.GetItemsInContainer() < 1)
      return;

   os << "#ROOMS\r\n";
   for (TArrayAsVectorIterator<TRoom *> it(TheRooms); it; it++)
      WriteRoom(it.Current(), os);
//      {
//      const TRoom *room = it.Current();
//
//      os << '#' << room->vnum << '\n';
//      os << room->name << "~\n";
//      os << room->description << "~\n";
//      os << "0 " << room->room_flags << ' ' << room->sector_type << '\n';
//   	for (int door = 0; door <= 5; door++ )
//         {
//    		if (room->is_exit[door])
//            {
//            os << 'D' << door << '\n';
//            os << room->exit[door].description << "~\n";
//            os << room->exit[door].keyword << "~\n";
//            os << exitinfo2locktype(room->exit[door].exit_info) << ' ' <<
//                   room->exit[door].key << ' ' << room->exit[door].vnum <<
//						 '\n';
//            }
//         }
//
//      for (TArrayAsVectorIterator<TExtraDescr *> it2(room->extra_descr); it2; it2++)
//         {
//         os << "E\n";
//         os << it2.Current()->keyword << "~\n";
//         os << it2.Current()->description << "~\n";
//         }
//
//      os << "S\n";
//		}

	os << "#0\r\n\r\n\r\n";
}

void TAreaDocument::WriteHelps(ostream& os)
{
	if (TheHelps.GetItemsInContainer() < 1)
      return;

   os << "#HELPS\r\n";
   for (TArrayAsVectorIterator<THelpData *> it(TheHelps); it; it++)
      WriteHelp(it.Current(), os);
   os << "0 $~\r\n\r\n\r\n";
}

void TAreaDocument::WriteMobs(ostream& os)
{
	if (TheMobiles.GetItemsInContainer() < 1)
      return;

   os << "#MOBILES\r\n";
   for (TArrayAsVectorIterator<TMobile *> it(TheMobiles); it; it++)
      WriteMobile(it.Current(), os);
//      {
//      const TMobile *mob = it.Current();
//
//      os << '#' << mob->vnum << '\n';
//      os << mob->player_name << "~\n";
//      os << mob->short_descr << "~\n";
//      os << mob->long_descr << "~\n";
//      os << mob->description << "~\n";
//
//      os << mob->act << ' ' << mob->affected_by << ' ' << mob->alignment <<
//          " S\n";
//      os << mob->level << ' ' << mob->hitroll << ' ' << mob->ac << ' ' <<
//         mob->hitnodice << 'd' << mob->hitsizedice << '+' << mob->hitplus <<
//         ' ' << mob->damnodice << 'd' << mob->damsizedice << '+' <<
//         mob->damplus << '\n';
//      os << mob->gold << ' ' << "0\n";
//      os << "8 8 " << mob->sex << '\n';
//
//      if (mob->mobprogs.GetItemsInContainer() > 0)
//         {
//         char szBuffer[256] = "Warning:  Stripping MOBprogs from ";
//         char szMobName[256];
//         mob->GetName(szMobName, sizeof szMobName);
//         strcat(szBuffer, szMobName);
//         MessageBox(0, szBuffer, "MAKE.ZONES.FAST", MB_ICONINFORMATION|MB_OK);
//         }
//      }

   os << "#0\r\n\r\n\r\n";
}

void TAreaDocument::WriteMobProgs(ostream& /*os*/)
{
//   os << "#MOBPROGS\n";
   // @@@ Should check if it is ok to write...  Should write
//   os << "S\n\n\n";
}

void TAreaDocument::WriteFlags(ostream& os, int32 flags)
{
   // @@@ 32?
   DWORD dw = 1;
   BOOL fFirst = TRUE;
   for (int i = 0; i < 31 && flags; i++, flags >>= 1, dw <<= 1)
      {
      if (flags&1)
         {
         if (fFirst)
            fFirst = FALSE;
         else
            os << '|';
         os << dw;
         }
      }

   if (fFirst)
      os << '0';
}

void TAreaDocument::WriteObjects(ostream& os)
{
	if (TheObjects.GetItemsInContainer() < 1)
      return;

   os << "#OBJECTS\r\n";
   for (TArrayAsVectorIterator<TObject *> it(TheObjects); it; it++)
      WriteObject(it.Current(), os);
//      {
//      const TObject *obj = it.Current();
//
//      os << '#' << obj->vnum << '\n';
//      os << obj->name << "~\n";
//      os << obj->short_descr << "~\n";
//      os << obj->description << "~\n";
//      os << "~\n";
//      os << obj->item_type << ' ';
//      WriteFlags(os, obj->extra_flags);
//      os << ' ';
//      WriteFlags(os, obj->wear_flags);
//      os << '\n';
//
//      for (int i = 0; i < nelems(obj->value); i++)
//         {
//         os << obj->value[i];
//         if (MuditorApp->CurrentMode == poundsApp::EnvyMode)
//            os << '~';
//#if 1 // @@@
//			if (atoi(obj->value[i].c_str()) == 0 && obj->value[i] != "0")
//            MessageBox(0, obj->value[i].c_str(), "Bad Value?", MB_ICONHAND|MB_OK);
//#endif
//         os << ' ';
//         }
//      os << '\n';
//
//      os << obj->weight << ' ' << obj->cost << " 0\n";
//
//      for (TArrayAsVectorIterator<TAffect *> it2(obj->affected); it2; it2++)
//         {
//         os << "A\n";
//         os << it2.Current()->location << ' ' <<
//               it2.Current()->modifier << '\n';
//         }
//
//      for (TArrayAsVectorIterator<TExtraDescr *> it3(obj->extra_descr); it3; it3++)
//         {
//         os << "E\n";
//         os << it3.Current()->keyword << "~\n";
//         os << it3.Current()->description << "~\n";
//         }
//	   }

   os << "#0\r\n\r\n\r\n";
}

void TAreaDocument::WriteResets(ostream& os)
{
	if (TheResets.GetItemsInContainer() < 1 &&
       TheMobResets.GetItemsInContainer() < 1)
      return;

	os << "#RESETS\r\n";
#if 0
   for (TArrayAsVectorIterator<TMobReset *> it2(TheMobResets); it2; it2++)
      WriteMobReset(it2.Current(), os);
#endif

   for (TArrayAsVectorIterator<TReset *> it(TheResets); it; it++)
      WriteReset(it.Current(), os);
//      {
//      const TReset *reset = it.Current();
//
//      os << reset->command;
//      switch (reset->command)
//         {
//         case '*':
//            os << reset->comment << '\n';
//            break;
//         case 'G':
//         case 'R':
//         case 'C':
//         case 'F':
//            os << " 1 ";
//            os << reset->arg1 << ' ' << reset->arg2 << "\t\t\t" <<
//                  reset->comment << '\n';
//            break;
//         default:
//            os << " 1 ";
//            os << reset->arg1 << ' ' << reset->arg2 << ' ' <<
//                  reset->arg3 << "\t\t\t" << reset->comment << '\n';
//            break;
//         }
//      }

   os << "S\r\n\r\n\r\n";
}

void TAreaDocument::WriteShops(ostream& os)
{
	if (TheShops.GetItemsInContainer() < 1)
      return;

   os << "#SHOPS\r\n";
	// Examine all shops.
   for (TArrayAsVectorIterator<TShopData *> it(TheShops); it; it++)
      WriteShop(it.Current(), os);
//      {
//      const TShopData *shop = it.Current();
//
//      os << shop->keeper << ' ';
//      for (int i = 0; i < MAX_TRADE; i++)
//         os << shop->buy_type[i] << ' ';
//      os << shop->profit_buy << ' ' << shop->profit_sell << ' ';
//      os << shop->open_hour << ' ' << shop->close_hour << ' ';
//      os << /*@@@shop->comment <<*/ '\n';
//      }
   os << "0\r\n\r\n\r\n";
}

void TAreaDocument::WriteSpecials(ostream& os)
{
   BOOL fFirst = TRUE;

   for (TArrayAsVectorIterator<TMobile *> it(TheMobiles); it; it++)
      {
      const TMobile *mob = it.Current();
      if (!mob->spec_fun.is_null())
         {
         if (fFirst)
            {
            os << "#SPECIALS\r\n";
            fFirst = FALSE;
            }

         os << "M  " << mob->vnum << ' ' << mob->spec_fun;
         if (mob->spec_comment.is_null())
            {
            if (!mob->short_descr.is_null())
               os << "\t\t\t" << mob->short_descr << "\r\n";
            }
         else
            {
            if (!mob->spec_comment.is_null() && mob->spec_comment[0] > ' ')
               os << "\t\t\t";
            os << mob->spec_comment << "\r\n";
            }
         }
      }

    if (!fFirst)
   	 os << "S\r\n\r\n\r\n";
}

/*
 * Troll-only
 *
 * #AREADATA
 * Name   { All } Locke    Newbie School~
 * Repop  A teacher pops in the room and says, 'Repop coming!'~
 * Recall 3001
 * Weather <weather data>
 * Timezone 12
 * Flags 4096
 * End
 */
void TAreaDocument::WriteAreaData(ostream& os)
{
   os << "#AREADATA\r\n";

   // Added as a quick fix (slay me!!)
   area_data.plane = area_data.plane + 1;

   os << "F " << area_data.area_flags << "\r\n";

   os << "P " << area_data.plane << "\r\n";

   os << "S\r\n\r\n";

   
 //  os << "Name " << name << "~\r\n";
 //  if (!area_data.repop.is_null())
 //     os << "Repop\t" << area_data.repop << "~\r\n";
 //  if (recall != -1)
 //     os << "#RECALL\r\n" << recall << "\r\n";
 //  if (area_data.fCustomWeather)
 //  os << "Weather\t" << area_data.Weather.MinTemp << ' ' <<
 //                        area_data.Weather.MaxTemp << ' ' <<
 //                        area_data.Weather.MinPrecip << ' ' <<
 //                        area_data.Weather.MaxPrecip << ' ' <<
 //                       area_data.Weather.MinWind << ' ' <<
 //                        area_data.Weather.MaxWind << ' ' <<
 //                      area_data.Weather.MinMagic << ' ' <<
 //                        area_data.Weather.MaxMagic << ' ' <<
 //                        area_data.Weather.MaxChange << "\r\n";
 //  os << "Timezone\t" << area_data.timezone << "\r\n";
 //  os << "Flags\t" << area_data.area_flags << "\r\n";
 //  os << "End\r\n\r\n";
}

#pragma argsused
void TAreaDocument::WriteHelp(THelpData *help, ostream& os)
{
   os << help->level << ' ' << help->keyword << "~\r\n" <<
         help->text << "~\r\n\r\n";
}

void TAreaDocument::WriteMobile(TMobile *mob, ostream& os)
{
   os << '#' << mob->vnum << "\r\n";
   os << mob->player_name << "~\r\n";
   os << mob->short_descr << "~\r\n";
   os << mob->long_descr << "\r\n~\r\n";    // (Add back on the crlf we stripped)
   trim(mob->description);
   os << mob->description << "\r\n~\r\n";
//      os << mob->description << "~\r\n";

   WriteFlags(os, mob->act);
   os << ' ';
   WriteFlags(os, mob->affected_by);
//      os << mob->act << ' ' << mob->affected_by << ' ' << mob->alignment <<
//          " S\r\n";
   os << ' ' << mob->alignment << " S\r\n";
   os << mob->level << ' ' << mob->hitroll << ' ' << mob->ac << ' ' <<
         mob->hitnodice << 'd' << mob->hitsizedice << '+' << mob->hitplus <<
         ' ' << mob->damnodice << 'd' << mob->damsizedice << '+' <<
         mob->damplus << "\r\n";
   os << mob->gold << ' ' << "0\r\n";
   os << "8 8 " << mob->sex << "\r\n";

   // Writing Race/Class

   if ( mob->race != 24 )
     os << "R " << mob->race << "\r\n";
	if ( mob->clss != 3 )
     os << "C " << mob->clss << "\r\n";

   if (MuditorApp->fMOBprograms)
      if (mob->mobprogs.GetItemsInContainer() > 0)
         {
         for (TArrayAsVectorIterator<TMobProgData*> it(mob->mobprogs); it; it++)
            {
            if (!it.Current()->MobFile.is_null())
               MessageBox(0, "Warning: mprog file being inlined", "M.Z.F.",
                  MB_ICONINFORMATION|MB_OK);
            os << '>' << it.Current()->type << ' ' << it.Current()->arglist <<
                  "~\r\n" << it.Current()->comlist << "~\r\n";
            }

         os << "|\r\n";
         }
#if 0
         {
         char szBuffer[256] = "Warning:  Stripping MOBprogs from ";
         char szMobName[256];
         mob->GetName(szMobName, sizeof szMobName);
         strcat(szBuffer, szMobName);
         MessageBox(0, szBuffer, "MAKE.ZONES.FAST", MB_ICONINFORMATION|MB_OK);
         }
#endif
}

#pragma argsused
void TAreaDocument::WriteMobProg(TMobProgData *mobprog, ostream& os)
{
   // @@@
}

void TAreaDocument::WriteObject(TObject *obj, ostream& os)
{
      os << '#' << obj->vnum << "\r\n";
      os << obj->name << "~\r\n";
      os << obj->short_descr << "~\r\n";
      os << obj->description << "~\r\n";
      os << "~\r\n";
      os << obj->item_type << ' ';
      WriteFlags(os, obj->extra_flags);
      os << ' ';
      WriteFlags(os, obj->wear_flags);
      os << "\r\n";

		int nMaxObjects = (MuditorApp->CurrentMode == poundsApp::RomMode) ? 5 : 4;
      for (int i = 0; i < nMaxObjects; i++)
         {
         if (MuditorApp->CurrentMode == poundsApp::EnvyMode)
            {
            switch ( obj->item_type )
               {
               default:
                  os << obj->value[i] << '~';
                  break;

               case ITEM_PILL:
               case ITEM_POTION:
               case ITEM_SCROLL:
                  if (i == 1 || i == 2 || i == 3)
                     os << SpellName(atoi(obj->value[i].c_str()), TRUE) << '~';
                  else
                     os << obj->value[i] << '~';
                  break;

               case ITEM_STAFF:
               case ITEM_WAND:
                  if (i == 3)
                     os << SpellName(atoi(obj->value[i].c_str()), TRUE) << '~';
                  else
                     os << obj->value[i] << '~';
                  break;
               }
         }
         else
            os << obj->value[i];

//			if (atoi(obj->value[i].c_str()) == 0 && obj->value[i] != "0")
//            MessageBox(0, obj->value[i].c_str(), "Bad Value?", MB_ICONHAND|MB_OK);
         os << ' ';
         }
      os << "\r\n";

      os << obj->weight << ' ' << obj->cost << " 0\r\n";

      for (TArrayAsVectorIterator<TAffect *> it2(obj->affected); it2; it2++)
         {
         os << "A\r\n";
         os << it2.Current()->location << ' ' <<
               it2.Current()->modifier << "\r\n";
         }

      for (TArrayAsVectorIterator<TExtraDescr *> it3(obj->extra_descr); it3; it3++)
         {
         os << "E\r\n";
         os << it3.Current()->keyword << "~\r\n";
         os << it3.Current()->description << "~\r\n";
         }
}

void TAreaDocument::WriteRoom(TRoom *room, ostream& os)
{
      os << '#' << room->vnum << "\r\n";
      os << room->name << "~\r\n";
      trim(room->description);
      os << room->description << "\r\n~\r\n";
//      os << "0 " << room->room_flags << ' ' << room->sector_type << "\r\n";
      os << "0 ";
      WriteFlags(os, room->room_flags);
      os << ' ' << room->sector_type << "\r\n";
   	for (int door = 0; door <= 5; door++ )
         {
    		if (room->is_exit[door])
            {
            os << 'D' << door << "\r\n";
            os << room->exit[door].description << "~\r\n";
            os << room->exit[door].keyword << "~\r\n";
            os << exitinfo2locktype(room->exit[door].exit_info) << ' ' <<
                   room->exit[door].key << ' ' << room->exit[door].vnum <<
						 "\r\n";
            }
         }

      for (TArrayAsVectorIterator<TExtraDescr *> it2(room->extra_descr); it2; it2++)
         {
         os << "E\r\n";
         os << it2.Current()->keyword << "~\r\n";
         os << it2.Current()->description << "~\r\n";
         }

      os << "S\r\n";
}

void TAreaDocument::WriteReset(TReset *reset, ostream& os, BOOL fChild)
{
   if (!fChild && strchr("GCFE", reset->command))
      return;

      if (reset->command != 'M')
         os << reset->command;
      switch (reset->command)
         {
         case '*':
            os << reset->comment << "\r\n";
            break;
         case 'C': // Random inv
         case 'F': // Random Eq
         case 'R': // Exit randomization
         case 'G': // Mob inv
            os << " 1 ";
            os << reset->arg1 << ' ' << reset->arg2;
            // Do we need to insert some white space?
            if (!reset->comment.is_null() && reset->comment[0] > ' ')
               os << "\t\t\t";
            os << reset->comment << "\r\n";
            break;
         case 'E': // Mob equip
            os << " 1 " << reset->arg1 << ' ' << reset->arg2 << ' ' <<
               reset->arg3;
            // Do we need to insert some white space?
            if (!reset->comment.is_null() && reset->comment[0] > ' ')
               os << "\t\t\t";
            os << reset->comment << "\r\n";
            break;
         case 'M': // WriteMobReset does M's
            WriteMobReset((TMobReset *) reset, os);
            break;
         default:
            os << " 1 ";
            os << reset->arg1 << ' ' << reset->arg2 << ' ' <<
                  reset->arg3;
            // Do we need to insert some white space?
            if (!reset->comment.is_null() && reset->comment[0] > ' ')
               os << "\t\t\t";
            os << reset->comment << "\r\n";
            break;
         }
}

void TAreaDocument::WriteMobReset(TMobReset *mres, ostream& os)
{
//   os << "*\r\nM 1 ";
   os << "M 1 ";
   os << mres->arg1 << ' ' << mres->arg2 << ' ' <<
         mres->arg3;

   // Do we need to insert some white space?
   if (!mres->comment.is_null() && mres->comment[0] > ' ')
      os << "\t\t\t";

   os << mres->comment << "\r\n";

   for (TArrayAsVectorIterator<TReset *> it(mres->Wields); it; it++)
      WriteReset(it.Current(), os, TRUE);
   for (TArrayAsVectorIterator<TReset *> it2(mres->Inventory); it2; it2++)
      WriteReset(it2.Current(), os, TRUE);
}

void TAreaDocument::WriteShop(TShopData *shop, ostream& os)
{
      os << shop->keeper << ' ';
      for (int i = 0; i < MAX_TRADE; i++)
         os << shop->buy_type[i] << ' ';
      os << shop->profit_buy << ' ' << shop->profit_sell << ' ';
      os << shop->open_hour << ' ' << shop->close_hour;
      if (!shop->comment.is_null() && shop->comment[0] > ' ')
         os << "\t\t\t";
      os << shop->comment << "\r\n";
}


static char *szClassNames[] = { "Mage", "Cleric", "Rogue", "Warrior", "Martial Artist",
	"Ranger", "Psionicist", "Monk", "Samurai" };

const char *TAreaDocument::ClassList(int n)
{
	if ( n < nelems(szClassNames))
   	return szClassNames[n];
   return 0;
}

static char *szRaceNames[] = { "Human", "Deep Gnome", "Drow", "Duergar", "Dwarf", "Elf",
	"Gnome", "Halfling", "Half-Elf", "Half-Orc", "Lizard Man", "Sprite", "Troglodyte",
   "Dragon", "Ogre", "Troll", "Kzinti", "Centaur", "Orc", "Giant", "Golem", "Cyborg",
   "Mobile", "Goblin", "Animal", "Griffon", "Gargoyle", "Demon", "Devil", "Doppleganger",
   "Elemental", "Ent", "Ghost", "Gith", "Harpy", "Insectazoid", "Merperson", "Illithid",
   "Minotaur", "Modron", "Myconid" };

const char *TAreaDocument::RaceList(int n)
{
	if ( n < nelems(szRaceNames))
   	return szRaceNames[n];
   return 0;
}

// @@@ Make part of class?
static char *szDamageTypes[] = { "hit", "slice", "stab", "slash", "whip",
      "claw", "blast", "pound", "crush", "puncture", "bite", "pierce", "chop" };

const char *TAreaDocument::DamageType(int n)
{
   if (n < nelems(szDamageTypes))
      return szDamageTypes[n];
   return 0;
}

static char *szContainerTrapTypes[] = { "[none]", "Poison Gas", "Poison Darts",
		"Poison Needle", "Gas Spores" };

const char *TAreaDocument::ContainerTrapType(int n)
{
   if ( n < nelems(szContainerTrapTypes))
   	return szContainerTrapTypes[n];
   return 0;
}

static char *szContainerFlags[] = { "[none]", "Closeable", "Pickproof", "",
		"Closed", "", "", "", "Locked" };

const char *TAreaDocument::ContainerFlags(int n)
{
  if ( n < nelems(szContainerFlags))
  		return szContainerFlags[n];
  return 0;
}

// @@@ Make part of class
   static char *szLiquidTypes[] = { "Water", "Beer", "Wine", "Ale",
      "Darkale", "Whisky", "Lemonade", "Firebreather", "Local Specialty",
      "Slime", "Milk", "Tea", "Coffee", "Blood", "Saltwater", "Cherry Cola",
      };

const char *TAreaDocument::LiquidType(int n)
{
   if (n < nelems(szLiquidTypes))
      return szLiquidTypes[n];
   return 0;
}

int TAreaDocument::MaxSpell()
{
   return 206;
}

// Flags:
//    None: Diku
//    1: Merc
//    2: TFC
//    3: Envy
struct SpellAndSlot { char *p; int n; int Flags; };
SpellAndSlot far SpellList[] = {
   { "[none]"               , -1,  1 },
   { "acid blast"           , 70,  1 },
   { "armor"                ,  1,  1 },

   { "bamf"						 , 300, 1 },
   { "barkskin"				 , 601, 1 },
   { "blindness"            ,  4,  1 },

   { "call lightning"       ,  6,  1 },
   { "cause critical"       , 63,  1 },
   { "cause light"          , 62,  1 },
   { "cause serious"        , 64,  1 },
   { "chain lightning"		 , 302, 1 },
   { "charm person"         ,  7,  1 },
   { "colour spray"         , 10,  1 },
   { "control weather"      , 11,  1 },
   { "create food"          , 12,  1 },
   { "create spring"        , 80,  1 },
   { "create water"         , 13,  1 },
   { "cure blindness"       , 14,  1 },
   { "cure critical"        , 15,  1 },
   { "cure disease"         ,501,  1 },
   { "cure light"           , 16,  1 },
   { "cure poison"          , 43,  1 },
   { "cure serious"         , 61,  1 },

   { "detect evil"          , 18,  1  },
   { "detect hidden"        , 44,  1 },
   { "detect invis"         , 19,  1 },
   { "detect magic"         , 20,  1 },
   { "detect poison"        , 21,  1 },
   { "dispel magic"         , 59,  1 },

   { "earthbind"				 ,628,  1 },
   { "earthquake"           , 23,  1 },
   { "enchant armor"			 ,301,  1 },
   { "enchant weapon"       , 24,  1 },
   { "energy drain"         , 25,  1 },

   { "faerie fire"          , 72,  1 },
   { "faerie fog"           , 73,  1 },
   { "fireball"             , 26,  1 },
   { "flamestrike"          , 65,  1 },
   { "fly"                  , 56,  1 },

   { "giant strength"		 , 39,  1 },

   { "heal"                 , 28,  1 },

   { "ice lance"				 ,603,  1 },
   { "identify"             , 53,  1 },
   { "infravision"          , 77,  1 },
   { "invis"                , 29,  1 },

   { "know alignment"       , 58,  1 },

   { "lightning bolt"       , 30,  1 },

   { "magic light"			 , 57,  1 },
   { "medicine"				 ,605,  1 },

   { "nourish"					 ,634,  1 },

   { "pass door"            , 74,  1 },
   { "poison"               , 33,  1 },
   { "prayer"				    ,606,  1 },
   { "protection"           , 34,  1 },

   { "refresh"              , 81,  1 },
   { "remove curse"         , 35,  1  },

   { "sanctuary"            , 36,  1 },
   { "sandstorm"				 ,611,  1 },
   { "shield"               , 67,  1 },
   { "sleep"                , 38,  1 },
   { "stone skin"           , 66,  1 },

   { "water breathing"		 ,632,  1 },
   { "weaken"               , 68,  1 },
   { "word of recall"       , 42,  1  },

   { "acid breath"          ,200,  1 }, // (Dragon spells are not player castable,
   { "fire breath"          ,201,  1 }, //  but can be on wands, etc)
   { "frost breath"         ,202,  1 },
   { "gas breath"           ,203,  1 },
   { "lightening breath"    ,204,  1 }



};

const char *TAreaDocument::SpellName(int nSlot, BOOL fForSave)
{
   // A -1 always means no spell when writing to the file
   if (fForSave && nSlot == -1)
      return "";

   // Give the application first crack at the spell, for overrides
   const char *p = TheMuditorApp->SpellName(nSlot);
   if (p)
      return p;

   // Look up in our list of spells for popular servers
   for (int i = 0; i < nelems(SpellList); i++)
      if (nSlot == SpellList[i].n)
         return SpellList[i].p;

   return 0;
}

const char *TAreaDocument::SpellIndex(int nNdx, int *nSlot)
{
   // Give the application first crack at the spell slot, for overrides
   if (nNdx < TheMuditorApp->nSlotsUsed)
      return TheMuditorApp->SpellIndex(nNdx, nSlot);
   nNdx -= TheMuditorApp->nSlotsUsed;

   if (nNdx < nelems(SpellList))
      {
      if (TheMuditorApp->CurrentMode == poundsApp::TFCMode)
         *nSlot = (SpellList[nNdx].Flags != 3) ? SpellList[nNdx].n : 0;
      else if (TheMuditorApp->CurrentMode == poundsApp::EnvyMode)
         *nSlot = (SpellList[nNdx].Flags != 2) ? SpellList[nNdx].n : 0;
      else // Merc
         *nSlot = (SpellList[nNdx].Flags < 2) ? SpellList[nNdx].n : 0;
      return SpellList[nNdx].p;
      }

   return 0;
}


int TAreaDocument::ClassList(const char *p)
{
   for (int i = 0; i <nelems(szClassNames); i++)
   	if ( strcmp(p, szClassNames[i]) == 0)
      	return i;

   MessageBox(0, "Warning:  Unknown Class converted to 'Warrior'",
   	"M.Z.F", MB_ICONHAND|MB_OK);
   return 3;
}

int TAreaDocument::RaceList(const char *p)
{
	for (int i = 0; i <nelems(szRaceNames); i++)
   	if ( strcmp(p, szRaceNames[i]) == 0 )
      	return i;

   MessageBox(0, "Warning:  Unknown Race converted to 'Animal'",
   	"M.Z.F", MB_ICONHAND|MB_OK);
   return 24;
}


int TAreaDocument::DamageType(const char *p)
{
   for (int i = 0; i < nelems(szDamageTypes); i++)
      if (strcmp(p, szDamageTypes[i]) == 0)
         return i;

   MessageBox(0, "Warning:  Unknown damage type converted to 'slash'",
      "M.Z.F", MB_ICONHAND|MB_OK);
   return 3;
}

int TAreaDocument::ContainerFlags(const char *p)
{
   for (int i = 0; i < nelems(szContainerFlags); i++)
      if (strcmp(p, szContainerFlags[i]) == 0)
      	return i;
   // else it's not closed or anything
   return 0;
}

int TAreaDocument::ContainerTrapType(const char *p)
{
   for (int i = 0; i < nelems(szContainerTrapTypes); i++)
     	if (strcmp(p, szContainerTrapTypes[i]) == 0 )
      	return i;

   //else it's not a trap
   return 0;
}

int TAreaDocument::SpellValue(const char *p)
{
   if (p[0] == '\0')
      return 0;

   // Give the app customization first crack at this spell type      
   int n = TheMuditorApp->SpellValue(p);
   if (n)
      return n;

   for (int i = 0; i < nelems(SpellList); i++)
      if (stricmp(p, SpellList[i].p) == 0)
         return SpellList[i].n;

   char szBuffer[512];
   wsprintf(szBuffer,
      "Warning:  Unknown spell type '%s' converted to '[none]'", p);
   MessageBox(0, szBuffer, "M.Z.F", MB_ICONHAND|MB_OK);
   return 0;
}

int TAreaDocument::LiquidVal(const char *p)
{
   for (int i = 0; i < nelems(szLiquidTypes); i++)
      if (strcmp(p, szLiquidTypes[i]) == 0)
         return i;

   MessageBox(0, "Warning:  Unknown liquid type converted to water",
      "M.Z.F", MB_ICONHAND|MB_OK);
   return 0;
}


bool TAreaDocument::Close ()
{
    BOOL result;

    result = TFileDocument::Close();

    // INSERT>> Your code here.
    if (IsDirty())
       MessageBox(0, "@@@Closing Dirty...", "M.Z.F.", MB_ICONINFORMATION|MB_OK);

    return result;
}

bool TAreaDocument::CanClose ()
{
//    BOOL result;
//
//    result = TFileDocument::CanClose();
//
//    // INSERT>> Your code here.
//    return result;

   // The original Expert generated code did not call FlushDoc if there was
   // more than one view.
   return GetDocManager().FlushDoc(*this);  // do the UI in the doc manager
}

