// This file is a continuating from areadoc.cpp which got to large
#include <owl\owlpch.h>
#pragma hdrstop

#include "areadoc.h"

extern struct race_type race_table[];

/*
 * Snarf a reset section.
 */
void TAreaDocument::load_resets( istream& is )
{
	 PReset pReset;
	 PMobReset pMobReset = 0;
	 char szBuffer[256];
	 PRoom pRoom;

	 if (name.is_null())
		 SyntaxError(is, "Loading resets", "There must be an #AREA section before the #RESETS section");

    for ( ; ; )
    {
    PExit pexit;
    PRoom pRoomIndex;
    char letter;

	if ( ( letter = fread_letter( is ) ) == 'S' )
	    break;

   if (letter == 'M')
      {
      if (pMobReset)
         TheMobResets.Add(pMobReset);
		pReset = pMobReset = new TMobReset(this);
      }
   else
		pReset = new TReset(this);

	if ( letter == '*' )
	{
	    pReset->comment = fread_to_eol( is );
       TheResets.Add(pReset);
	    continue;
	}

	pReset->command	= letter;
	/* if_flag */	  fread_number( is, "Ignoring reset if-flag" );
	pReset->arg1	= (sh_int) fread_number( is, "Reading reset arg1" );
	pReset->arg2	= (sh_int) fread_number( is, "Reading reset arg2" );
	pReset->arg3	= ( letter == 'G' || letter == 'R' ||
                      letter == 'C' || letter == 'F' )
			    ? (sh_int) 0 : (sh_int) fread_number( is, "Reading reset arg3" );
	pReset->comment = fread_to_eol( is );

   TObject *obj;
	switch ( letter )
	{
	default:
//	    bug( "Load_resets: bad command '%c'.", letter );
		 wsprintf(szBuffer,
          "TAreaDocument::load_resets(): Unknown reset '%c'.  Try to continue?",
          letter);
		 if (MessageBox(0, szBuffer, "MAKE.ZONES.FAST", MB_ICONQUESTION|MB_YESNO)
              != IDYES)
          THROW( TXOwl("") );
	    break;

	case 'M':
	    /*pMobile =*/ get_mob_index  ( pReset->arg1 );
       pRoom = get_room_index ( pReset->arg3 );
       if (pRoom)
			 pRoom->MobResets.Add(pMobReset);
	    break;

	case 'O':
//	    get_obj_index  ( pReset->arg1 );
       pRoom = get_room_index ( pReset->arg3 );
       if (pRoom)
          pRoom->ObjResets.Add(pReset);
	    break;

	case 'P':
//	    get_obj_index  ( pReset->arg1 );
//	    get_obj_index  ( pReset->arg3 );
		 break;

	case 'E':
	    obj = get_obj_index  ( pReset->arg1 );
		 if (obj)
          obj->count = pReset->arg2;
       if (pMobReset)
          pMobReset->Wields.Add(pReset);
	    break;

	case 'G':
	    obj = get_obj_index  ( pReset->arg1 );
       if (obj)
			 obj->count = pReset->arg2;
       if (pMobReset)
          pMobReset->Inventory.Add(pReset);
	    break;

   case 'C':
   case 'F':
       if (pMobReset)
          pMobReset->Inventory.Add(pReset);
	    break;

	case 'D':
	    pRoomIndex = get_room_index( pReset->arg1 );

       // ECS added test for valid room to allow editor to handle zones that
       // reset doors in other zones.
       if (pRoomIndex)
			 {
          char szError[512];

          // Check for a valid door direction (N, S, E, W, U, D)
          if ( pReset->arg2 < 0 || pReset->arg2 > 5 )
             {
             pRoomIndex->GetName(szBuffer, sizeof szBuffer);
             wsprintf(szError, "%s: Door reset specifies invalid door %d.",
                szBuffer, pReset->arg2);
//             bug(szError, 0);
             SyntaxError(is, "Reading D reset", szBuffer);
             /* NOTREACHED */
             }

          // Check for a valid door position (Open, Closed, Locked)
          const char *szExitNames[] =
             { "North", "East", "South", "West", "Up", "Down" };
   	    if ( pReset->arg3 < 0 || pReset->arg3 > 2 )
             {
             pRoomIndex->GetName(szBuffer, sizeof szBuffer);
             wsprintf(szError, "%s: %s door is set into invalid position %d."
                "  Make it a door?", szBuffer, szExitNames[pReset->arg2],
					 pReset->arg3);
				 SyntaxError(is, "Checking for valid door position", szError);
				 }

			 // Verify that the room knows about the door
			 if ( !IS_SET( pRoomIndex->exit[pReset->arg2].exit_info, EX_ISDOOR ) )
				 {
				 pRoomIndex->GetName(szBuffer, sizeof szBuffer);
				 wsprintf(szError,
					 "%s: %s door is set into position, but isn't a door."
					 "  Make it a door?", szBuffer, szExitNames[pReset->arg2]);
				 if (MessageBox(0, szError, "MAKE.ZONES.FAST",
														  MB_ICONQUESTION|MB_YESNO) == IDYES)
                {
                pRoomIndex->exit[pReset->arg2].exit_info = EX_ISDOOR;
                pRoomIndex->is_exit[pReset->arg2] = TRUE;
                }
             }

          // @@@ Check for an already exiting reset for this door...
          pRoomIndex->exit[pReset->arg2].InitialPosition = pReset;
          pRoomIndex->exit[pReset->arg2].nInitialPosition = pReset->arg3;
          }

		 break;

	case 'R':
		 pRoomIndex		= get_room_index( pReset->arg1 );

	    if ( pReset->arg2 < 0 || pReset->arg2 > 6 )
			 SyntaxError(is, "Reading R randomize reset", "R reset specifies invalid number of exits");

		 if (pRoomIndex)
			 pRoomIndex->RandomDoor = pReset;

		 break;
	}

	TheResets.Add(pReset);
	}

	if (pMobReset)
		TheMobResets.Add(pMobReset);
}



/*
 * Snarf a room section.
 */
void TAreaDocument::load_rooms( istream& is )
{
	 PRoom pRoomIndex;
	 char szBuffer[256];

	 if (name.is_null())
	 {
	 SyntaxError(is, "Reading #ROOMS", "There must be an #AREA section before the #ROOMS section");
	 }

	 for ( ; ; )
	 {
	char letter;
	sh_int  vnum;
	int  door;

	letter				= fread_letter( is );
	if ( letter != '#' )
	{
		 SyntaxError(is, "Looking for a room", "Room did not begin with # character");
	}

	vnum				= (sh_int) fread_number( is, "Reading room VNUM" );
	if ( vnum == 0 )
		 break;

	if ( get_room_index( vnum ) )
	{
       wsprintf(szBuffer, "Room vnum #%d duplicated", vnum);
		 SyntaxError(is, "Reading rooms", szBuffer);
	}

	pRoomIndex = new TRoom(this);
	pRoomIndex->vnum		= vnum;
	pRoomIndex->name		= fread_string( is, "Reading room name" );
	pRoomIndex->description		= fread_string( is, "Reading room description" );
	/* Area number */		  fread_number( is, "Ignoring room area number" );   /* Unused */
   if (MuditorApp->CurrentMode == poundsApp::RomMode)
   	pRoomIndex->room_flags		= fread_flag( is, "Reading room flags" );
   else
	   pRoomIndex->room_flags		= fread_number( is, "Reading room flags" );
	pRoomIndex->sector_type		= (sh_int) fread_number( is, "Reading room sector type" );
	for ( door = 0; door <= 5; door++ )
	    pRoomIndex->is_exit[door] = FALSE;

	for ( ; ; )
	{
	    letter = fread_letter( is );

	    if ( letter == 'S' || letter == 's' )
          break;

		 if ( letter == 'D' )
		 {
		 int        locks;

		door = (sh_int) fread_number( is, "Reading door number" );
		if ( door < 0 || door > 5 )
		{
          wsprintf(szBuffer, "Room vnum #%d has bad door number", vnum);
          SyntaxError(is, "Reading exit", szBuffer);
		}

      PExit pexit = &pRoomIndex->exit[door];
      pRoomIndex->is_exit[door] = TRUE;
		pexit->description	= fread_string( is, "Reading exit description" );
		pexit->keyword		= fread_string( is, "Reading exit keyword" );
		pexit->exit_info	= 0;
		locks			= (sh_int) fread_number( is, "Reading 'locks' value" );
		pexit->key		= (sh_int) fread_number( is, "Reading door key vnum" );
		pexit->vnum		= (sh_int) fread_number( is, "Reading exit 'to room'" );

		switch ( locks )
		{
		case 1: pexit->exit_info = EX_ISDOOR;                    break;
		case 2: pexit->exit_info = EX_ISDOOR | EX_PICKPROOF;     break;
		case 3: pexit->exit_info = EX_ISDOOR | EX_BASHPROOF;     break;
		case 4: pexit->exit_info = EX_ISDOOR | EX_PICKPROOF
										 | EX_BASHPROOF;                 break;
		case 5: pexit->exit_info = EX_ISDOOR | EX_PASSPROOF;     break;
		case 6: pexit->exit_info = EX_ISDOOR | EX_PICKPROOF
										 | EX_PASSPROOF;                 break;
		case 7: pexit->exit_info = EX_ISDOOR | EX_BASHPROOF
		                         | EX_PASSPROOF;                 break;
		case 8: pexit->exit_info = EX_ISDOOR | EX_PICKPROOF
		                         | EX_BASHPROOF | EX_PASSPROOF;  break;
		}

	    }
	    else if ( letter == 'E' )
	    {
      PExtraDescr ed;

      ed = new TExtraDescr();
		ed->keyword		= fread_string( is, "Reading room extra description keyword" );
		ed->description		= fread_string( is, "Reading room extra description" );
      pRoomIndex->extra_descr.Add(ed);
	    }

	    else
	    {
       wsprintf(szBuffer, "Vnum #%d has a flag that is not D, E, or S: %c",
         vnum, letter);
       SyntaxError(is, "Loading room extras and doors", szBuffer);
	    }
	}

    if (nNextRoomVNum < vnum)
		 {
       // First room seen?
       if (nNextRoomVNum == -1)
			 {
          if (nNextObjVNum == -1)
             nNextObjVNum = vnum;
          if (nNextMobVNum == -1)
             nNextMobVNum = vnum;
          }

       nNextRoomVNum = vnum;
       }
    TheRooms.Add(pRoomIndex);
    strcpy(szLastSuccess, pRoomIndex->name.c_str());
    }

    return;
}



/*
 * Snarf a shop section.
 */
void TAreaDocument::load_shops( istream& is )
{
	 PShopData pShop;

	 for ( ; ; )
	 {
	PMobile pMobIndex;
	int iTrade;

	pShop = new TShopData(this);
	pShop->keeper		= (sh_int) fread_number( is, "Reading shopkeeper vnum" );
	if ( pShop->keeper == 0 )
		 break;
	for ( iTrade = 0; iTrade < MAX_TRADE; iTrade++ )
		 pShop->buy_type[iTrade] = (sh_int) fread_number( is, "Reading shopkeeper trade types" );
	pShop->profit_buy	= (sh_int) fread_number( is, "Reading shopkeeper profit buy %" );
	pShop->profit_sell	= (sh_int) fread_number( is, "Reading shopkeeper profit sell %" );
	pShop->open_hour	= (sh_int) fread_number( is, "Reading shopkeeper open hour" );
	pShop->close_hour	= (sh_int) fread_number( is, "Reading shopkeeper close hour" );
	pShop->comment = 			  fread_to_eol( is );
	pMobIndex		= get_mob_index( pShop->keeper );
	// We check for a valid shopkeeper, as some zones (eg OfCol) make mobs in
	// other zones shopkeepers...
	if (pMobIndex)
		pMobIndex->pShop	= pShop;

	TheShops.Add(pShop);
	 }

    return;
}



/*
 * Snarf spec proc declarations.
 */
void TAreaDocument::load_specials( istream& is )
{
   char szBuffer[256];

   for ( ; ; )
      {
      PMobile pMobIndex;
	   char letter;

	   switch ( letter = fread_letter( is ) )
	      {
	      default:
//	         bug( "Load_specials: letter '%c' not must be *, M, or S.", letter );
            wsprintf(szBuffer, "Expecting *, M, or S, got '%c'", letter);
            SyntaxError(is, "Reading a special entry", szBuffer);

         case 'S':
				return;

	      case '*':
				// @@@ We lose all '*' comments in #SPECIALS
      	   fread_to_eol( is );
	         break;

         case 'M':
	         pMobIndex = get_mob_index((sh_int) fread_number ( is, "Reading mob vnum" ) );
	         pMobIndex->spec_fun = fread_word( is, "Reading spec-fun name" );
       	   pMobIndex->spec_comment = fread_to_eol( is );
            break;
	      }
      }
}

#define UMIN( a, b )		( ( a ) < ( b ) ? ( a ) : ( b ) )
#define UMAX( a, b )		( ( a ) > ( b ) ? ( a ) : ( b ) )

/*
 * Troll-only
 *
 * Snarf an 'area' header line.   Check this format.  MUCH better.
 * Add fields too.
 *
 * Function taken from 'The Isles' code.  Walker.
 *
 * #AREADATA
 * Name   { All } Locke    Newbie School~
 * Repop  A teacher pops in the room and says, 'Repop coming!'~
 * Recall 3001
 * Weather <weather data>
 * Timezone 12
 * Flags 4096
 * End
 */          //Works now for AVATAR
void TAreaDocument::load_troll_area( istream& is )
{
//    AREA_DATA *pArea;
	 char buf[ 128 ];
    int /*min, max,*/ n;	/* For James' new weather system.  Walker */
    char letter;
    for ( ; ; )
    {
      letter = fread_letter(is);

      if ( letter == 'S' || letter == 's' )
        {
        	fUsesAreaData = TRUE;
         return;
        }
      else if ( letter == 'F' || letter == 'f' )
      	{
         area_data.area_flags = fread_number( is, "Reading Area Flags" );
         }
      else if ( letter == 'P' || letter == 'p' )
       	{
         area_data.plane = (((sh_int) fread_number(is, "Reading Area Plane" )) - 1);
         // QUICK FIX (slay me!)
         }
      else {
			wsprintf(buf, "Unknown #AREADATA member: %c.  MEMBER WILL NOT SAVE!",
            letter);
         MessageBox(0, buf, "M.Z.F.", MB_ICONHAND|MB_OK);
		     } /* default */

      } /* for */
}


/*
 * Translate all room exits from virtual to real.
 * Has to be done after all rooms are read in.
 * Check for bad or suspicious reverse exits.
 */
void TAreaDocument::fix_exits( void )
{
//		 EXIT_DATA       *pexit;
       PExit pexit, pexit_rev;
//		 EXIT_DATA       *pexit_rev;
//		 ROOM_INDEX_DATA *pRoomIndex;
//		 ROOM_INDEX_DATA *to_room;
       PRoom pRoomIndex, to_room;
//                 char             buf     [ MAX_STRING_LENGTH ];
//    extern const int              rev_dir [ ];
//		 int              iHash;
		 int              door;

//    for ( iHash = 0; iHash < MAX_KEY_HASH; iHash++ )
//    {
//	for ( pRoomIndex  = room_index_hash[iHash];
//	      pRoomIndex;
//	      pRoomIndex  = pRoomIndex->next )
      for ( TArrayAsVectorIterator<TRoom *> it(TheRooms); it; it++)
//      for ( TArrayAsVectorIterator<TVNumObj *> it(TheRooms); it; it++)
	{
       pRoomIndex = it.Current();
//       pRoomIndex = (TRoom *) it.Current();
//	    bool fexit;
       BOOL fexit;

	    fexit = FALSE;
	    for ( door = 0; door <= 5; door++ )
	    {
//		if ( ( pexit = pRoomIndex->exit[door] ) )
		if ( ( pRoomIndex->is_exit[door] ) )
		{
		    pexit = &pRoomIndex->exit[door];
		    fexit = TRUE;
		    if ( pexit->vnum <= 0 )
			pexit->to_room = NULL;
		    else
			pexit->to_room = get_room_index( pexit->vnum );
		}
		 }


       // I'm not sure what this was for...but it was screwing things up!!
//		 if ( !fexit )
//		SET_BIT( pRoomIndex->room_flags, ROOM_NO_MOB );
	}
//    }

#if 0 // This checks for exit coherence, there is no need...
//    for ( iHash = 0; iHash < MAX_KEY_HASH; iHash++ )
//    {
//	for ( pRoomIndex  = room_index_hash[iHash];
//	      pRoomIndex;
//	      pRoomIndex  = pRoomIndex->next )
		for ( it.Restart(); it; it++)
	{
		 for ( door = 0; door <= 5; door++ )
		 {
//		if (   ( pexit     = pRoomIndex->exit[door]       )
		if (   ( pRoomIndex->is_exit[door]       )
			 && ( to_room   = pRoomIndex->exit[door].to_room               )
//		    && ( pexit_rev = to_room->exit[rev_dir[door]] )
			 && ( to_room->is_exit[rev_dir[door]] )
//		    &&   pexit_rev->to_room != pRoomIndex )
			 &&   to_room->exit[rev_dir[door]].to_room != pRoomIndex )
		{
			 wsprintf( buf, "Fix_exits: %d:%d -> %d:%d -> %d.",
				 pRoomIndex->vnum, door,
				 pRoomIndex->exit[door].vnum,    rev_dir[door],
				 to_room->exit[rev_dir[door]].vnum );
		    bug( buf, 0 );
		}
	    }
	}
//    }
#endif
}

/*
 * Read a letter from a file.
 */
char TAreaDocument::fread_letter( istream& is )
{
    int c;

    do {
      c = is.get();
    } while ( isspace( c ) );

    return (char) c;
}



/*
 * Read a number from a file.
 */
int32 TAreaDocument::fread_number( istream& is, const char *operation )
{
    int c;
    int32  number;
    BOOL sign;

    // Skip leading spaces
    do {
	    c = is.get();
    } while ( isspace( c ) );

    number = 0;

    // Set sign == TRUE if there is a leading - sign
    sign   = FALSE;
    if ( c == '+' )
    {
	    c = is.get();
    }
    else if ( c == '-' )
    {
		 sign = TRUE;
	    c = is.get();
    }

    // If we don't have a digit here we are not at a number.
    if ( !isdigit( c ) )
    {
#if 0
       is.putback( (char) c );
       string s = fread_to_eol( is );
       MessageBox(0, s.c_str(), "fread_number() given this mess", MB_OK);
#endif
       SyntaxError(is, operation, "Bad number format");
    }

    // Read in the number
    while ( isdigit(c) )
    {
	    number = number * 10 + c - '0';
	    c = is.get();
    }

    // If it was a negative number, make it negative
    if ( sign )
	    number = 0 - number;

	 // If the number is | combined with another number, read the next number
    if ( c == '|' )
	    number += fread_number( is, operation );
    else if ( c != ' ' )
       is.putback( (char) c);

    return number;
}



/*
 * Read and allocate space for a string from a file.
 * These strings are read-only and shared.
 * Strings are hashed:
 *   each string prepended with hash pointer to prev string,
 *   hash code is simply the string length.
 * This function takes 40% to 50% of boot-up time.
 */
string TAreaDocument::fread_string( istream& is, const char *operation )
{
    char *plast;
    int c;
    char szBuffer[MAX_STRING_LENGTH];

    plast = szBuffer;
//    plast = top_string + sizeof( char * );
//    if ( plast > &string_space [ MAX_STRING - MAX_STRING_LENGTH ] )
//    {
//	bug( "Fread_string: MAX_STRING %d exceeded.", MAX_STRING );
//	exit( 1 );
//    }

    /*
     * Skip blanks.
     * Read first char.
     */
    do
    {
	c = is.get();
    }
    while ( isspace( c ) );

    if ( ( *plast++ = (char) c ) == '~' )
//	return &str_empty[0];
       return "";

    for ( ;; )
    {
	/*
	 * Back off the char type lookup,
	 *   it was too dirty for portability.
	 *   -- Furey
	 */
	switch ( *plast = (char) is.get() )
	{
	default:
	    plast++;
	    break;

	case EOF:
//	    bug( "Fread_string: EOF", 0 );
       SyntaxError(is, operation, "Unexpected End of File");
//	    exit( 1 );
//	    break;

//	case '\n':
//	    plast++;
//	    *plast++ = '\r';
//	    break;
//
//	case '\r':
//	    break;

	case '~':
	    plast++;
#if 0
	    {
		union
		{
		    char *	pc;
		    char	rgc[sizeof( char * )];
		} u1;
		int ic;
		int iHash;
		char *pHash;
		char *pHashPrev;
		char *pString;

		plast[-1] = '\0';
		iHash     = UMIN( MAX_KEY_HASH - 1, plast - 1 - top_string );
		for ( pHash = string_hash[iHash]; pHash; pHash = pHashPrev )
		{
		    for ( ic = 0; ic < sizeof( char * ); ic++ )
			u1.rgc[ic] = pHash[ic];
		    pHashPrev = u1.pc;
		    pHash    += sizeof(char *);

		    if ( top_string[sizeof( char * )] == pHash[0]
			&& !strcmp( top_string+sizeof( char * )+1, pHash+1 ) )
			return pHash;
		}

		if ( fBootDb )
		{
		    pString             = top_string;
		    top_string		= plast;
		    u1.pc		= string_hash[iHash];
		    for ( ic = 0; ic < sizeof( char * ); ic++ )
			pString[ic] = u1.rgc[ic];
		    string_hash[iHash]  = pString;

		    nAllocString += 1;
		    sAllocString += top_string - pString;
		    return pString + sizeof( char * );
		}
		else
		{
		    return str_dup( top_string + sizeof( char * ) );
		}
	    }
#else
		plast[-1] = '\0';
		    return string(szBuffer);
#endif
	}
    }
}



/*
 * Read to end of line (for comments).
 */
string TAreaDocument::fread_to_eol( istream& is )
{
    char szBuffer[MAX_STRING_LENGTH];
    char *plast = szBuffer;
    int c;

    do
    {
	c = is.get();
   *plast++ = (char) c;
    }
    while ( c != '\n' && c != '\r' );

    plast[-1] = '\0';
    do
    {
	c = is.get();
    }
    while ( c == '\n' || c == '\r' );

//    ungetc( c, fp );
    is.putback( (char) c);
    return string(szBuffer);
}



/*
 * Read one word (into static buffer).
 */
char *TAreaDocument::fread_word( istream& is, const char *operation )
{
    static char  word [ MAX_INPUT_LENGTH ];
    char *pword;
    int cEnd;

    do
    {
	    cEnd = is.get();
    }
    while ( isspace( cEnd ) );

    if ( cEnd == '\'' || cEnd == '"' )
    {
	pword   = word;
    }
    else
    {
	word[0] = (char) cEnd;
	pword   = word+1;
	cEnd    = ' ';
    }

    for ( ; pword < word + MAX_INPUT_LENGTH; pword++ )
    {
	*pword = (char) is.get();
	if ( cEnd == ' ' ? isspace( *pword ) : *pword == cEnd )
	{
	    if ( cEnd == ' ' )
//		ungetc( *pword, fp );
      is.putback(*pword);
	    *pword = '\0';
	    return word;
	}
    }

    SyntaxError(is, operation, "Word too long");
//    bug( "Fread_word: word too long.", 0 );
//    exit( 1 );
    return NULL;
}

/*
 * Read ROM-style flags
 */
int32 TAreaDocument::fread_flag( istream& is, const char *operation )
{
   int32 number = 0;
   int c;

   do {
	    c = is.get();
   } while ( isspace( c ) );

   if (!isdigit(c)) {
	   while (('A' <= c && c <= 'Z') || ('a' <= c && c <= 'z')) {
	       number += flag_convert((char) c);
	       c = is.get();
	   }
   }

   while (isdigit(c)) {
	   number = number * 10 + c - '0';
	   c = is.get();
   }

   if (c == '|')
	   number += fread_flag( is, operation );
   else if  ( c != ' ')
      is.putback( (char) c);

   return number;
}

int32 TAreaDocument::flag_convert(char letter) const
{
   int32 bitsum = 0;
   char i;

   if ('A' <= letter && letter <= 'Z') {
	   bitsum = 1;
	   for (i = letter; i > 'A'; i--)
	      bitsum *= 2;
   } else if ('a' <= letter && letter <= 'z') {
	   bitsum = 67108864L; /* 2^26 */
	   for (i = letter; i > 'a'; i --)
	      bitsum *= 2;
   }

   return bitsum;
}

// Big concern here is that the old beginning # and the new beginning # might
// overlap.
void TAreaDocument::Renumber( sh_int nNewBase )
{
//    sh_int new_vnum;
   sh_int nNewRoomBase = nNewBase, nNewObjBase = nNewBase, nNewMobBase = nNewBase;

   // First, assign the new # to all of the objects
   for ( TArrayAsVectorIterator<TRoom *> itRoom(TheRooms); itRoom; itRoom++)
      itRoom.Current()->new_vnum = nNewRoomBase++;

   for ( TArrayAsVectorIterator<TMobile *> itMob(TheMobiles); itMob; itMob++)
      itMob.Current()->new_vnum = nNewMobBase++;

   for ( TArrayAsVectorIterator<TObject *> itObj(TheObjects); itObj; itObj++)
      itObj.Current()->new_vnum = nNewObjBase++;

   // Now, update all of the exits:
   for ( itRoom.Restart(); itRoom; itRoom++)
      for (int i = 0; i < 6; i++)
         if (itRoom.Current()->is_exit[i])
            {
            TRoom *r = itRoom.Current()->exit[i].to_room;
            if (r)
               {
               itRoom.Current()->exit[i].vnum = r->new_vnum;
               // ECS Added key renumbering 10/5/94
               if (itRoom.Current()->exit[i].key > 0)
                  {
                  TObject *pKey = get_obj_index(itRoom.Current()->exit[i].key);
                  if (pKey)
                     itRoom.Current()->exit[i].key = pKey->new_vnum;
                  }
               }
            }

   // Now, update all of the resets
   TMobile *pMobile;
   TRoom   *pRoom;
   TObject *pObj;
   for (TArrayAsVectorIterator<TReset *> itRes(TheResets); itRes; itRes++)
       {
       TReset *pReset = itRes.Current();
       switch (pReset->command) {

       case 'M': // Place mobile
	       pMobile = get_mob_index( pReset->arg1 );
          if (pMobile)
             pReset->arg1 = pMobile->new_vnum;
          pRoom = get_room_index( pReset->arg3 );
          if (pRoom)
             pReset->arg3 = pRoom->new_vnum;
          break;

       case 'O':	// Put an object in the room
	       pObj = get_obj_index( pReset->arg1 );
          if (pObj)
             pReset->arg1 = pObj->new_vnum;

          pRoom = get_room_index( pReset->arg3 );
          if (pRoom)
             pReset->arg3 = pRoom->new_vnum;
    	    break;

	    case 'P':  // Put an object in an object
	       pObj = get_obj_index( pReset->arg1 );
          if (pObj)
             pReset->arg1 = pObj->new_vnum;

	       pObj = get_obj_index( pReset->arg3 );
          if (pObj)
             pReset->arg1 = pObj->new_vnum;
   	    break;

	    case 'E':	// Equip a mobile
	    case 'G':  // Give a mobile some inventory
	       pObj = get_obj_index( pReset->arg1 );
          if (pObj)
             pReset->arg1 = pObj->new_vnum;
   	    break;

//       case 'C':
//       case 'F':

       case 'D': // Set a door
       case 'R': // Randomize a room
   	    pRoom = get_room_index( pReset->arg1 );
          if (pRoom)
             pReset->arg1 = pRoom->new_vnum;
   	    break;
	    }
       }

   // Now, update the shops
   for ( TArrayAsVectorIterator<TShopData *> itShop(TheShops); itShop; itShop++)
      {
	   pMobile = get_mob_index( itShop.Current()->keeper );
      if (pMobile)
         itShop.Current()->keeper = pMobile->new_vnum;
      }

   // We have now updated everything that is safe to update (we haven't touched
   // resets FOR OTHER ZONES or Slash's spec-fun cargo in long description.)

   // Complete the renumbering
   for ( itRoom.Restart(); itRoom; itRoom++)
      itRoom.Current()->vnum = itRoom.Current()->new_vnum;

   for ( itMob.Restart(); itMob; itMob++)
      itMob.Current()->vnum = itMob.Current()->new_vnum;

   for ( itObj.Restart(); itObj; itObj++)
      itObj.Current()->vnum = itObj.Current()->new_vnum;

   // Make the views redraw themselves
   SetDirty(TRUE);
   NotifyViews(vnRevert, TRUE);

//   // Success!!!
//   MessageBox(0, "Ok", "MAKE.ZONES.FAST", MB_ICONINFORMATION|MB_OK);
}

void TAreaDocument::SyntaxError(istream& is, const char *operation, const char *msg)
{
   // Go back to the beginning of the area file and count the number of newlines
   // it took to get here
   int iLine = 0;
   if (!is.bad())
      {
      long lPos = is.tellg();
      is.seekg(0);
   	for ( ; is.tellg() < lPos; iLine++ )
	   	while ( is.get() != '\n' && !is.bad())
		      ;
      }

   char szBuffer[512];
   wsprintf(szBuffer, "Section: #%s\r\nLast Success: %s\r\nOperation: %s\r\n"
      "Line: %d\r\n Bug: %s", szSection, szLastSuccess, operation, iLine, msg);
   THROW( TXOwl(szBuffer) );
}

/* returns race number, for ROM */
sh_int TAreaDocument::race_lookup (const string& name) const
{
   for (sh_int race = 0; race_table[race].name != NULL; race++)
      {
	   if (!name.is_null() && LOWER(name[0]) == LOWER(race_table[race].name[0])
	      && !str_prefix(name.c_str(), race_table[race].name))
	    return race;
      }

   return 0;
}

/*
 * Compare strings, case insensitive, for prefix matching.
 * Return TRUE if astr not a prefix of bstr
 *   (compatibility with historical functions).
 */
BOOL TAreaDocument::str_prefix( const char *astr, const char *bstr ) const
{
    if ( astr == NULL )
    {
//	bug( "Strn_cmp: null astr.", 0 );
	return TRUE;
    }

    if ( bstr == NULL )
    {
//	bug( "Strn_cmp: null bstr.", 0 );
	return TRUE;
    }

    for ( ; *astr; astr++, bstr++ )
    {
	if ( LOWER(*astr) != LOWER(*bstr) )
		 return TRUE;
    }

	 return FALSE;
}

///* returns material number */
//int TAreaDocument::material_lookup (const char *name)
//{
//    // The original ROM version is empty, as well...
//    return 0;
//}
