/*  Project pounds

    Copyright  1994. All Rights Reserved.

    SUBSYSTEM:    pounds.apx Application
    FILE:         mobdia.cpp
    AUTHOR:       Slash


    OVERVIEW
    ========
    Source file for implementation of TMobDialog (TDialog).
*/

#include <owl\owlpch.h>
#pragma hdrstop

#include "mobdia.h"
#include "shopdia.h"
#include "appdia.h"
#include "mobprog.h"

//
// Build a response table for all messages/commands handled
// by the application.
//
DEFINE_RESPONSE_TABLE1(TMobDialog, TDialog)
//{{TMobDialogRSP_TBL_BEGIN}}
    EV_BN_CLICKED(IDC_PREV, CmPrev),
    EV_BN_CLICKED(IDC_NEXT, CmNext),
    EV_BN_CLICKED(IDOK, CmOk),
    EV_BN_CLICKED(IDC_MALE, CmSexChanged),
    EV_BN_CLICKED(IDC_FEMALE, CmSexChanged),
    EV_BN_CLICKED(IDC_NEUTER, CmSexChanged),
    EV_CBN_SELCHANGE(IDC_SPECFUN, CmSpecChanged),
    EV_BN_CLICKED(IDHELP, CmHelp),
    EV_BN_CLICKED(IDC_ASSISTALWAYS, CmAlwaysAssist),
    EV_BN_CLICKED(IDC_RESETS, CmViewResets),
    EV_BN_CLICKED(IDC_MOBPROG, CmMobProg),
//{{TMobDialogRSP_TBL_END}}
END_RESPONSE_TABLE;


// Make part of class
struct SpecialNameAndIcon { char *szName; int nMode; int nIcon; };
static SpecialNameAndIcon Specials[nelems(((TMobDialog*)0)->hMobSpecIcon)] = {
   { "[none]",           0, IDI_SPEC_NONE },
   { "breath_any",       0, IDI_SPEC_BREATH },
   { "breath_acid",      0, IDI_SPEC_BREATH },
   { "breath_fire",      0, IDI_SPEC_BREATH },
   { "breath_frost",     0, IDI_SPEC_BREATH },
   { "breath_gas",       0, IDI_SPEC_BREATH },
   { "breath_lightning", 0, IDI_SPEC_BREATH },
   { "cast_adept",       0, IDI_SPEC_ADEPT },
   { "cast_cleric",      0, IDI_SPEC_CLERIC },
   { "cast_judge",       0, IDI_SPEC_JUDGE },

   { "cast_mage",        0, IDI_SPEC_MAGE },
   { "cast_undead",      0, IDI_SPEC_UNDEAD },
   { "fido",             0, IDI_SPEC_FIDO },
   { "guard",            0, IDI_SPEC_GUARD },
   { "janitor",          0, IDI_SPEC_JANITOR },
   { "poison",           0, IDI_SPEC_POISON },
   { "thief",            0, IDI_SPEC_THIEF },
   { "assasin",			 0, IDI_UNKNOWN_SPEC },
   { "cast_psion",		 0, IDI_UNKNOWN_SPEC },
   { "doppleganger",		 0, IDI_UNKNOWN_SPEC },
   { "kzin",			    0, IDI_UNKNOWN_SPEC },
   { "Buddha",				 0, IDI_UNKNOWN_SPEC },
   { "teacher",			 0, IDI_UNKNOWN_SPEC },
   { "white guard",		 0, IDI_UNKNOWN_SPEC },
   { "kungfu poison",	 0, IDI_UNKNOWN_SPEC },
   { "animate dead",     0, IDI_UNKNOWN_SPEC },
   { "stomp em",			 0, IDI_UNKNOWN_SPEC },

   { "",                -1, IDI_UNKNOWN_SPEC }
};

//{{TMobDialog Implementation}}


static TMobDialogXfer TMobDialogData;

TMobDialog::TMobDialog (TWindow* parent, const TMobile *mob, int nNdx,
        TResId resId, TModule* module):
    TDialog(parent, resId, module), WorkMob( *mob )
{
//{{TMobDialogXFER_USE}}
    NameList = new TEdit(this, IDC_NAMELIST, 255);
    ShortDescr = new TEdit(this, IDC_SHORT_DESCR, 255);
    LongDescr = new TEdit(this, IDC_LONG_DESCR, 255);
    LookDescr = new TEdit(this, IDC_LOOK, 4096);
    SpecFun = new TComboBox(this, IDC_SPECFUN, 1);
    Level = new TThacoEdit(this, IDC_LEVEL, 255);
    Male = new TRadioButton(this, IDC_MALE, 0);
    Female = new TRadioButton(this, IDC_FEMALE, 0);
    Neuter = new TRadioButton(this, IDC_NEUTER, 0);
    Align = new TEdit(this, IDC_ALIGN, 255);
    Sentinel = new TCheckBox(this, IDC_SENTINEL, 0);
    Scavenger = new TCheckBox(this, IDC_SCAVENGER, 0);
    Aggresive = new TCheckBox(this, IDC_AGGRESSIVE, 0);
    StayArea = new TCheckBox(this, IDC_STAY_AREA, 0);
    Wimpy = new TCheckBox(this, IDC_WIMPY, 0);
    TrainPCs = new TCheckBox(this, IDC_TRAIN, 0);
    PracticePCs = new TCheckBox(this, IDC_PRACTICE, 0);
    AggieAll = new TCheckBox(this, IDC_AGGIE_ALL, 0 );
    IsHealer = new TCheckBox(this, IDC_IS_HEALER, 0 );
    IsUndead = new TCheckBox(this, IDC_UNDEAD, 0 );
    StayTerrain = new TCheckBox(this, IDC_STAY_TERRAIN, 0 );
    Invisible = new TCheckBox(this, IDC_INVISIBLE, 0);
    DetectInvis = new TCheckBox(this, IDC_DETECTINVIS, 0);
    DetectHidden = new TCheckBox(this, IDC_DETECT_HIDDEN, 0);
    FaerieFire = new TCheckBox(this, IDC_FAERIE_FIRE, 0);
    ProtEvil = new TCheckBox(this, IDC_PROT_FROM_EVIL, 0);
    Sneak = new TCheckBox(this, IDC_SNEAK, 0);
    Hide = new TCheckBox(this, IDC_HIDE, 0);
    Flying = new TCheckBox(this, IDC_FLYING, 0);
    PassDoor = new TCheckBox(this, IDC_PASSDOOR, 0);
    Sanctuary = new TCheckBox(this, IDC_SANCT, 0);
    Blind = new TCheckBox(this, IDC_BLIND, 0 );
    MoveHidden = new TCheckBox(this, IDC_MOVEHIDE, 0 );
    VNum = new TStatic(this, IDC_VNUM, 255);
    SexIcon = new TStatic(this, IDC_SEXICON, 255);
    SpecIcon = new TStatic(this, IDC_SPECICON, 255);
    HitPoints = new TStatic(this, IDC_HP, 255);
    ArmorClass = new TStatic(this, IDC_AC, 255);
    Thac0 = new TStatic(this, IDC_THAC0, 255);
    AssistChance = new TCheckBox(this, IDC_ASSIST, 0);
    AssistAlways = new TCheckBox(this, IDC_ASSISTALWAYS, 0);
    AssistSame = new TCheckBox(this, IDC_ASSISTSAME, 0);
    Class = new TComboBox(this, IDC_CLASS, 1 );
    Race = new TComboBox(this, IDC_RACE, 1 );

    SetTransferBuffer(&TMobDialogData);
//{{TMobDialogXFER_USE_END}}

    // INSERT>> Your constructor code here.
    TheActualMobile = mob;
    nIndex = nNdx;

    char szBuffer[32], szSpecInfo[64];
    for (int i = 0; i < MAX_CUSTOM_SPEC; i++)
       {
       wsprintf(szBuffer, "specfun%d", i);
       GetProfileString("MAKE.ZONES.FAST", szBuffer, "", szSpecInfo,
          sizeof szSpecInfo);
       if (szSpecInfo[0])
          szCustomSpec[i] = strcpy(new char[strlen(szSpecInfo)+1], szSpecInfo);
       else
          szCustomSpec[i] = 0;
       }

    VNum->EnableTransfer();
    HitPoints->EnableTransfer();
    ArmorClass->EnableTransfer();
    Thac0->EnableTransfer();
    Class->EnableTransfer();
    Race->EnableTransfer();
//    Level->SetValidator(new TThacoValidator(this));
    LoadTransfer();

    for (i = 0; i < nelems(hMobSpecIcon); i++)
       hMobSpecIcon[i] = GetApplication()->LoadIcon(
          MAKEINTRESOURCE(Specials[i].nIcon));

    static int nSexIcon[] = { IDI_NEUTER, IDI_MALE, IDI_FEMALE };

    for (i = 0; i < nelems(hMobSexIcon); i++)
       hMobSexIcon[i] = GetApplication()->LoadIcon(
          MAKEINTRESOURCE(nSexIcon[i]));
}


TMobDialog::~TMobDialog ()
{
    Destroy();

    // INSERT>> Your destructor code here.
    for (int i = 0; i < nelems(hMobSpecIcon); i++)
       DestroyIcon(hMobSpecIcon[i]);

    for (i = 0; i < nelems(hMobSexIcon); i++)
       DestroyIcon(hMobSexIcon[i]);

    for (i = 0; i < MAX_CUSTOM_SPEC; i++)
       delete[] szCustomSpec[i];
}


void TMobDialog::CmPrev ()
{
    // INSERT>> Your code here.
    const TMobile *m;
    if ((m = WorkMob.AreaDoc->GetMobileFromIndex(nIndex-1)) != 0)
       {
   	 TransferData(tdGetData);
       SaveTransfer();
       nIndex--;
       LoadAnotherMobile(m);
       return;
       }

    MessageBeep(-1);
}


void TMobDialog::CmNext ()
{
    // INSERT>> Your code here.
    const TMobile *m;
    if ((m = WorkMob.AreaDoc->GetMobileFromIndex(nIndex+1)) != 0)
       {
   	 TransferData(tdGetData);
       SaveTransfer();
       nIndex++;
       LoadAnotherMobile(m);
       return;
       }

    MessageBeep(-1);
}

// @@@ Make part of class
#define UMAX( a, b )		( ( a ) > ( b ) ? ( a ) : ( b ) )
/*
 * Simple linear interpolation.
 */
int interpolate( int level, int value_00, int value_xx )
{
    // @@@ I think this needs to be 47 for EnvyMud...
    return value_00 + level * ( value_xx - value_00 ) / 32;
}

void TMobDialog::LoadTransfer()
{
    // load class/race

    LoadClassRace();
    
	 strcpy(TMobDialogData.NameList, WorkMob.player_name.c_str());
	 strcpy(TMobDialogData.ShortDescr, WorkMob.short_descr.c_str());
	 strcpy(TMobDialogData.LongDescr, WorkMob.long_descr.c_str());
	 strcpy(TMobDialogData.LookDescr, WorkMob.description.c_str());

    TMobDialogData.SpecFun.Clear();

    // Add the spec funs specific to this server type
    BOOL fKnownSpec = FALSE;
    for (int i = 0; i < nelems(Specials); i++)
       if (Specials[i].nMode == 0 ||
           Specials[i].nMode == (int) WorkMob.AreaDoc->GetMode())
       {
       BOOL fSelected;
       if (WorkMob.spec_fun.is_null())
          fSelected = (i == 0);
       else
          fSelected = (WorkMob.spec_fun == string("spec_") + Specials[i].szName);

       fKnownSpec |= fSelected;
       TMobDialogData.SpecFun.AddStringItem(Specials[i].szName, i, fSelected);
       }

    // Add customized spec-funs
    for (i = 0; i < MAX_CUSTOM_SPEC; i++)
       if (szCustomSpec[i])
          {
          BOOL fSelected = (WorkMob.spec_fun == string("spec_") + szCustomSpec[i]);
          fKnownSpec |= fSelected;
          TMobDialogData.SpecFun.AddStringItem(szCustomSpec[i], nelems(Specials)-1, fSelected);
          }

    // If this mobile has an unknown spec fun, add that too
    if (!fKnownSpec)
       TMobDialogData.SpecFun.AddStringItem(WorkMob.spec_fun.c_str(),
          nelems(Specials)-1, TRUE);

    wsprintf(TMobDialogData.Level, "%d", WorkMob.level);
    TMobDialogData.Male = (WorkMob.sex == SEX_MALE);
    TMobDialogData.Female = (WorkMob.sex == SEX_FEMALE);
    TMobDialogData.Neuter = (WorkMob.sex == SEX_NEUTRAL);
    wsprintf(TMobDialogData.Align, "%d", WorkMob.alignment);

    // Action flags
    TMobDialogData.Sentinel = IS_SET(WorkMob.act, ACT_SENTINEL);
    TMobDialogData.Scavenger = IS_SET(WorkMob.act, ACT_SCAVENGER);
    TMobDialogData.Aggresive = IS_SET(WorkMob.act, ACT_AGGRESSIVE);
    TMobDialogData.StayArea = IS_SET(WorkMob.act, ACT_STAY_AREA);
    TMobDialogData.Wimpy = IS_SET(WorkMob.act, ACT_WIMPY);
    TMobDialogData.TrainPCs = IS_SET(WorkMob.act, ACT_TRAIN);
    TMobDialogData.PracticePCs = IS_SET(WorkMob.act, ACT_PRACTICE);
    TMobDialogData.AggieAll = IS_SET(WorkMob.act, ACT_AGGIE_ALL);
    TMobDialogData.IsHealer = IS_SET(WorkMob.act, ACT_IS_HEALER);
    TMobDialogData.IsUndead = IS_SET(WorkMob.act, ACT_UNDEAD);
    TMobDialogData.StayTerrain = IS_SET(WorkMob.act, ACT_STAY_TERRAIN);

    if (WorkMob.AreaDoc->GetMode() == poundsApp::EnvyMode)
       {
       }
    else if (WorkMob.AreaDoc->GetMode() == poundsApp::TFCMode)
       {
       }

    // Affected by
    TMobDialogData.Invisible = IS_SET(WorkMob.affected_by, AFF_INVISIBLE);
    TMobDialogData.DetectInvis = IS_SET(WorkMob.affected_by, AFF_DETECT_INVIS);
    TMobDialogData.DetectHidden = IS_SET(WorkMob.affected_by, AFF_DETECT_HIDDEN);
    TMobDialogData.FaerieFire = IS_SET(WorkMob.affected_by, AFF_FAERIE_FIRE);
    TMobDialogData.Sneak = IS_SET(WorkMob.affected_by, AFF_SNEAK);
    TMobDialogData.Hide = IS_SET(WorkMob.affected_by, AFF_HIDE);
    TMobDialogData.Sanctuary = IS_SET(WorkMob.affected_by, AFF_SANCTUARY);
    TMobDialogData.Flying = IS_SET(WorkMob.affected_by, AFF_FLYING);
    TMobDialogData.PassDoor = IS_SET(WorkMob.affected_by, AFF_PASS_DOOR);
    TMobDialogData.Blind = IS_SET(WorkMob.affected_by, AFF_BLIND);
    TMobDialogData.MoveHidden = IS_SET(WorkMob.affected_by, AFF_MOVEHIDE);
    if (WorkMob.AreaDoc->GetMode() == poundsApp::EnvyMode) // @@@ Make these vis
       {

       }
    else if (WorkMob.AreaDoc->GetMode() == poundsApp::TFCMode)
       {

       }

    wsprintf(TMobDialogData.VNum, "%d", WorkMob.vnum);
}

void TMobDialog::SetThac0(int nLevel)
{
/*
    Stats are calculated thusly:
    min_hp = (level - 1) * 8 + (level-1) * (level - 1) / 4
    max_hp = (level + 1) * 8 + (level+1) * (level + 1)
    ac = interpolate(level, 100, -100)
    thac0 = interpolate(level, 20, 0)
    min_dam = (level - 1) / 2
    max_dam = (level + 1) * 3 / 2
*/
    int minlevel = UMAX(1, nLevel -1), maxlevel = nLevel + 1;
    int min_hp = minlevel * 8 + minlevel * minlevel / 4;
    int max_hp = maxlevel * 8 + maxlevel * maxlevel;
    int ac = interpolate(nLevel, 100, -100);
    int thac0 = interpolate(nLevel, 20, 0);

    wsprintf(TMobDialogData.HitPoints, "%d - %d", min_hp, max_hp);
    wsprintf(TMobDialogData.ArmorClass, "%d", ac);
    wsprintf(TMobDialogData.Thac0, "%d", thac0);

    HitPoints->SetText(TMobDialogData.HitPoints);
    ArmorClass->SetText(TMobDialogData.ArmorClass);
    Thac0->SetText(TMobDialogData.Thac0);
}

void TMobDialog::SaveTransfer()
{

    // Saves Class/Race
    SaveClassRace();

    WorkMob.player_name = TMobDialogData.NameList;
	 WorkMob.short_descr = TMobDialogData.ShortDescr;
	 WorkMob.long_descr = TMobDialogData.LongDescr;
	 WorkMob.description = TMobDialogData.LookDescr;

    char szBuffer[128];
    TMobDialogData.SpecFun.GetSelString(szBuffer, sizeof szBuffer);
    if (strcmp(szBuffer, "[none]") == 0)
       WorkMob.spec_fun = "";
    else
       {
       if (memcmp(szBuffer, "spec_", 5) == 0)
          WorkMob.spec_fun = szBuffer;
       else
          {
          WorkMob.spec_fun = "spec_";
          WorkMob.spec_fun += szBuffer;
          }
       }

	 WorkMob.level = (sh_int) atoi(TMobDialogData.Level);

    if (TMobDialogData.Male) 			WorkMob.sex = SEX_MALE;
    else if (TMobDialogData.Female) WorkMob.sex = SEX_FEMALE;
    else if (TMobDialogData.Neuter) WorkMob.sex = SEX_NEUTRAL;

    WorkMob.alignment = (sh_int) atoi(TMobDialogData.Align);

    // Action flags
    SET_RESET_BIT(WorkMob.act, TMobDialogData.Sentinel, ACT_SENTINEL);
    SET_RESET_BIT(WorkMob.act, TMobDialogData.Scavenger, ACT_SCAVENGER);
    SET_RESET_BIT(WorkMob.act, TMobDialogData.Aggresive, ACT_AGGRESSIVE);
    SET_RESET_BIT(WorkMob.act, TMobDialogData.StayArea, ACT_STAY_AREA);
    SET_RESET_BIT(WorkMob.act, TMobDialogData.Wimpy, ACT_WIMPY);
    SET_RESET_BIT(WorkMob.act, TMobDialogData.TrainPCs, ACT_TRAIN);
    SET_RESET_BIT(WorkMob.act, TMobDialogData.PracticePCs, ACT_PRACTICE);
    SET_RESET_BIT(WorkMob.act, TMobDialogData.AggieAll, ACT_AGGIE_ALL);
    SET_RESET_BIT(WorkMob.act, TMobDialogData.IsHealer, ACT_IS_HEALER);
    SET_RESET_BIT(WorkMob.act, TMobDialogData.IsUndead, ACT_UNDEAD);
    SET_RESET_BIT(WorkMob.act, TMobDialogData.StayTerrain, ACT_STAY_TERRAIN);

    if (WorkMob.AreaDoc->GetMode() == poundsApp::EnvyMode)
       {
       }
    else if (WorkMob.AreaDoc->GetMode() == poundsApp::TFCMode)
       {

       }

    // Affected by
    SET_RESET_BIT(WorkMob.affected_by, TMobDialogData.Invisible, AFF_INVISIBLE);
    SET_RESET_BIT(WorkMob.affected_by, TMobDialogData.DetectInvis, AFF_DETECT_INVIS);
    SET_RESET_BIT(WorkMob.affected_by, TMobDialogData.DetectHidden, AFF_DETECT_HIDDEN);
    SET_RESET_BIT(WorkMob.affected_by, TMobDialogData.FaerieFire, AFF_FAERIE_FIRE);
    SET_RESET_BIT(WorkMob.affected_by, TMobDialogData.Sneak, AFF_SNEAK);
    SET_RESET_BIT(WorkMob.affected_by, TMobDialogData.Hide, AFF_HIDE);
    SET_RESET_BIT(WorkMob.affected_by, TMobDialogData.Sanctuary, AFF_SANCTUARY);
    SET_RESET_BIT(WorkMob.affected_by, TMobDialogData.Flying, AFF_FLYING);
    SET_RESET_BIT(WorkMob.affected_by, TMobDialogData.PassDoor, AFF_PASS_DOOR);
    SET_RESET_BIT(WorkMob.affected_by, TMobDialogData.Blind, AFF_BLIND);
    SET_RESET_BIT(WorkMob.affected_by, TMobDialogData.MoveHidden, AFF_MOVEHIDE);
    if (WorkMob.AreaDoc->GetMode() == poundsApp::EnvyMode)
       {
       }
    else  if (WorkMob.AreaDoc->GetMode() == poundsApp::TFCMode)
       {
       }

    WorkMob.AreaDoc->MobileSave(WorkMob, nIndex);
}


void TMobDialog::SaveClassRace()
{
   char szBuffer1[64];
   int32 nVal1;
   char szBuffer2[64];
   int32 nVal2;
   char szBuffer3[64];
   char szBuffer4[64];

   Class->GetText(szBuffer1, sizeof szBuffer1);
   nVal1 = TAreaDocument::ClassList(szBuffer1);
   Race->GetText(szBuffer2, sizeof szBuffer2);
   nVal2 = TAreaDocument::RaceList(szBuffer2);

   wsprintf( szBuffer3, "%ld", nVal1 );
   WorkMob.clas = szBuffer3;

   wsprintf( szBuffer4, "%ld", nVal2 );
   WorkMob.race = szBuffer4;
}

void TMobDialog::LoadClassRace()
{
	int32 nVal1;
   int32 nVal2;
   const char *szText;
   const char *szText2;

   Class->Clear();
   Race->Clear();

   nVal1 = atol( WorkMob.clas.c_str());
   nVal2 = atol( WorkMob.race.c_str());

   int n = 0;
   szText = TAreaDocument::ClassList(n);
   while (szText)
   {
      Class->AddString(szText);
      if ( n == nVal1 )
         Class->SetText(szText);
      n++;
      szText = TAreaDocument::ClassList(n);
   }

   int i = 0;

   szText2 = TAreaDocument::RaceList(i);
   while (szText2)
   {
   	Race->AddString(szText2);
      if ( i == nVal2 )
      	Race->SetText(szText);
      i++;
      szText2 = TAreaDocument::RaceList(i);
   }

}







void TMobDialog::LoadAnotherMobile(const TMobile *m)
{
   TheActualMobile = m;
   WorkMob = *m;
   LoadTransfer();
   TransferData(tdSetData);
   CmSexChanged();
   CmSpecChanged();
   SetThac0(WorkMob.level);
   CmAlwaysAssist();
}


void TMobDialog::CmOk ()
{
    // INSERT>> Your code here.
  	 TransferData(tdGetData);
    SaveTransfer();
    TDialog::CmOk();
}


void TMobDialog::CmMakeKeeper ()
{
    // INSERT>> Your code here.
    if (nIndex == -1)
       {
       MessageBox("Sorry, you cannot currently make a mobile a shopkeeper "
          "until you have written him to the database by clicking OK on him.",
          "MAKE.ZONES.FAST", MB_ICONHAND|MB_OK);
       return;
       }


    int nShopIndex = WorkMob.AreaDoc->GetShopIndexFromMobileVNum(WorkMob.vnum);
    // @@@ This is kinda messed up, TAreaDocument should be doing this...
    TShopData* shop;
    if (nShopIndex < 0)
       {
       if (MessageBox("Make this mobile a shopkeeper?",
            "MAKE.ZONES.FAST", MB_ICONQUESTION|MB_YESNO) != IDYES)
          return;
       else
          {
          shop = WorkMob.AreaDoc->MakeNewShop();
          shop->keeper = WorkMob.vnum;
          }
       }
    else
       shop = WorkMob.AreaDoc->TheShops[nShopIndex];

    TShopDialog(WorkMob.AreaDoc->GetDocManager().GetApplication()->GetMainWindow(),
       WorkMob.AreaDoc, shop, nShopIndex, TRUE).Execute();
    if (nShopIndex < 0)
       delete shop;
//    WorkMob.AreaDoc->EditItem(4, nShopIndex);
}


void TMobDialog::CmSexChanged ()
{
    // INSERT>> Your code here.
    int nCurrentSex = SEX_NEUTRAL;
    if (Male->GetCheck() == BF_CHECKED)
       nCurrentSex = SEX_MALE;
    else if (Female->GetCheck() == BF_CHECKED)
       nCurrentSex = SEX_FEMALE;
    SexIcon->SendMessage(STM_SETICON, (WPARAM) hMobSexIcon[nCurrentSex], 0);
}


void TMobDialog::CmSpecChanged ()
{
    // INSERT>> Your code here.
    int n = SpecFun->GetSelIndex();
    if (n >= 0)
      {
      n = (int) SpecFun->GetItemData(n);
      if (n >= 0 && n < nelems(hMobSpecIcon))
         SpecIcon->SendMessage(STM_SETICON, (WPARAM) hMobSpecIcon[n], 0);
      }
}


void TMobDialog::SetupWindow ()
{
    TDialog::SetupWindow();

    // INSERT>> Your code here.
//    LookDescr->SendMessage(WM_SETFONT, (WPARAM) GetStockObject(ANSI_FIXED_FONT), 0);
    LookDescr->SendMessage(WM_SETFONT,
       (WPARAM) (HFONT) WorkMob.AreaDoc->GetMZF()->hFixedFont, 0);
    CmAlwaysAssist ();

    // Envy-only acts and affects
 //   Gamble->EnableWindow(WorkMob.AreaDoc->GetMode() == poundsApp::EnvyMode);

 

    /* TFC-only acts and affects
    BOOL fTFC = WorkMob.AreaDoc->GetMode() == poundsApp::TFCMode;
    SuperWimpy->EnableWindow(fTFC);
//    AssistChance->EnableWindow(fTFC);
    AssistAlways->EnableWindow(fTFC);
//    AssistSame->EnableWindow(fTFC);
    ProtGood->EnableWindow(fTFC);
//@@@    NoSleep->EnableWindow(fTFC);
    NoSummon->EnableWindow(fTFC);
    NoCharm->EnableWindow(fTFC);
    AntiMagic->EnableWindow(fTFC);
//@@@    FireShield->EnableWindow(fTFC);
    ImprovedInvis->EnableWindow(fTFC);
    Invulnerable->EnableWindow(fTFC);
    NoSteal->EnableWindow(fTFC);      */

    CmSexChanged();
    CmSpecChanged();
    SetThac0(WorkMob.level);
}

#if 0
//==============================================================================
BOOL TThacoValidator::IsValidInput(char far* p, BOOL suppressFill)
{
   mobdia->SetThac0();
   return TValidator::IsValidInput(p, suppressFill);
}
#endif

DEFINE_RESPONSE_TABLE1(TThacoEdit, TEdit)
  EV_WM_KILLFOCUS,
END_RESPONSE_TABLE;

void TThacoEdit::EvKillFocus(HWND hWndGetFocus)
{
   GetText(TMobDialogData.Level, sizeof TMobDialogData.Level);
   mobdia->SetThac0(atoi(TMobDialogData.Level));
   TEdit::EvKillFocus(hWndGetFocus);
}


void TMobDialog::CmHelp ()
{
    // INSERT>> Your code here.
    poundsApp::HelpState = WinHelp(HelpFileName, HELP_CONTEXT, IDD_MOBDIA);
}


void TMobDialog::CmAlwaysAssist ()
{
    // INSERT>> Your code here.
    BOOL f = (WorkMob.AreaDoc->GetMode() == poundsApp::TFCMode) &&
             (AssistAlways->GetCheck() != BF_CHECKED);

    AssistChance->EnableWindow(f);
    AssistSame->EnableWindow(f);
}


void TMobDialog::CmViewResets ()
{
    // INSERT>> Your code here.
    TResetRoomDialog (this, WorkMob.AreaDoc, WorkMob.vnum).Execute();
}


void TMobDialog::CmMobProg ()
{
    // INSERT>> Your code here.
    TMobProgDia(this, &WorkMob, WorkMob.mobprogs).Execute();
}

