#ifndef _OBJMERC_H
#define _OBJMERC_H
//#include <classlib\listimp.h>
#include <classlib\arrays.h>

typedef short sh_int;
typedef long  int32;
#define MAX_STRING_LENGTH	 4096
#define MAX_INPUT_LENGTH	  160

#define nelems(a) (sizeof (a)/sizeof (a)[0])

extern const char HelpFileName[];

// @@@;
class TAreaDocument;
class TRoom;
class TReset;

//
// class TPointer
// ----- --------
//
template<class T> class TSafePointerBase {
  public:
//    T&   operator  *() {return *P;}
//    T&   operator  *() { if (P==0) THROW(TXOwl("Bad Deref *")); return *P;}
    T&   operator  *();
         operator T*() {return P;}
    int  operator  !() {return P==0;}
    void operator  ~() {P = 0;}
//    void operator delete(void* p) { ((TSafePointerBase<T>*)p)->P=0;}
//    void operator delete(void* p) { delete ((TSafePointerBase<T>*)p)->P;}
    void do_delete() { delete P; }
  protected:
    TSafePointerBase(T* pointer) : P(pointer){}
//   ~TPointerBase() {delete P;}
    TSafePointerBase() : P(0){}
    T* P;
  private:
//    void* operator new(size_t){return 0;}  // prohibit use of new
};

template<class T> class TMercPointer : public TSafePointerBase<T> {
  public:
    TMercPointer() : TSafePointerBase<T>(){}
    TMercPointer(T* pointer) : TSafePointerBase<T>(pointer){}
    T* operator =(T* src) {/*delete P;*/ return P = src;}
    T* operator =(const TMercPointer<T>& src)
                          {/*delete P;*/ return P = src.P;}
//    T* operator->() { if (P==0) THROW(TXOwl("Bad deref ->")); return P;}
    T* operator->();
};

template<class T> T& TSafePointerBase<T>::operator *()
{
   if (P==0)
      THROW(TXOwl("Bad Deref *"));
   return *P;
}

template<class T> T* TMercPointer<T>::operator->()
{
   if (P==0)
      THROW(TXOwl("Bad deref ->"));
   return P;
}

class TMercObj {
public:
   TMercObj(TAreaDocument *doc);
   const virtual TMercObj *GetName(char *lpOut, int nOutSize) const = 0;

   TAreaDocument *AreaDoc;
};

// Help table types.
class THelpData: public TMercObj {
public:
   THelpData(TAreaDocument *doc);
   const virtual TMercObj *GetName(char *lpOut, int nOutSize) const;
   int operator==(const THelpData&) const;

   sh_int level;
   string keyword;
   string text;
};
typedef TMercPointer<THelpData> PHelpData;

// Shop types.
#define MAX_TRADE	 5
class TShopData: public TMercObj {
public:
    TShopData(TAreaDocument *doc);
    TShopData(const TShopData&);
    const virtual TMercObj *GetName(char *lpOut, int nOutSize) const;
    int operator==(const TShopData&) const;

    sh_int 	keeper;			/* Vnum of shop keeper mob	*/
    sh_int 	buy_type [ MAX_TRADE ];	/* Item types shop will buy	*/
    sh_int 	profit_buy;		/* Cost multiplier for buying	*/
    sh_int 	profit_sell;		/* Cost multiplier for selling	*/
    sh_int 	open_hour;		/* First opening hour		*/
    sh_int 	close_hour;		/* First closing hour		*/
    string  comment;
};
typedef TMercPointer<TShopData> PShopData;

// An affect.
class TAffect /*: public TMercObj*/ {
public:
//    TAffect(TAreaDocument *doc);
    TAffect() { location = modifier = 0; }
    TAffect(const TAffect&);
    int operator==(const TAffect&) const;
//    int32 		type;
//    int32 		duration;
    sh_int 		location;
    sh_int 		modifier;
//    int32		bitvector;
//    bool       deleted;
};
typedef TMercPointer<TAffect> PAffect;

// ACT bits for mobs.
// Used in #MOBILES.
#define ACT_IS_NPC		   1		/* Auto set for mobs	*/
#define ACT_SENTINEL		   2		/* Stays in one room	*/
#define ACT_SCAVENGER		4		/* Picks up objects	*/
#define ACT_AGGIE_ALL		8     // AVATAR
#define ACT_AGGRESSIVE	  32		/* Attacks PC's		*/
#define ACT_STAY_AREA	  64		/* Won't leave area	*/
#define ACT_WIMPY		    128		/* Flees when hurt	*/
#define ACT_TRAIN		    512		/* Can train PC's	*/
#define ACT_PRACTICE		 1024		/* Can practice PC's	*/
#define ACT_IS_HEALER	 2048    // AVATAR
#define ACT_UNDEAD		 4096    // AVATAR
#define ACT_LIVE_CORPSE  8192    // AVATAR
#define ACT_SOUL_STEALER 16384   // AVATAR
#define ACT_CANDLE_GIVER 32768L  // AVATAR
#define ACT_STAY_TERRAIN 65536L  // AVATAR
#define ACT_SMITH		    131072L // AVATAR



// Bits for 'affected_by'.
// Used in #MOBILES.
#define AFF_BLIND		            1
#define AFF_INVISIBLE            2  /*Mob is invisible*/
#define AFF_DETECT_INVIS         8  /*Mob can see invisible players*/
#define AFF_DETECT_HIDDEN       32  /*Mob can see 'hidden' players (thief skill)*/
#define AFF_SANCTUARY          128  /*Mob takes half damage from opponent.*/
#define AFF_FAERIE_FIRE        256  /*Should not normally be used.*/
#define AFF_PROTECT_EVIL      8192  /*Damage from an Evil source is reduced by 25%*/
#define AFF_SNEAK            32768L /*Can not see mob entering/exiting a room*/
#define AFF_HIDE             65536L /*Mob is hidden.  Player needs detect hidden*/
												/*or other means to see a hidden mob*/
#define AFF_SLEEP		       131072L    /* NON-TFC */
#define AFF_FLYING          524288L    /*Mob can fly.*/
#define AFF_PASS_DOOR       1048576L   /*Mob can enter/exit through closed doors.*/
#define AFF_MOVEHIDE			 8388608L
#define AFF_GIANT_STR       33554432L
#define AFF_RESTING		    67108864L
#define AFF_BIOFEEDBACK     134217728L
#define AFF_ADREN_RUSH		 268435456L
#define AFF_FAST_REGEN      536870912L



// Sex.
// Used in #MOBILES.
#define SEX_NEUTRAL		      0
#define SEX_MALE		         1
#define SEX_FEMALE		      2


// Item types.
// Used in #OBJECTS.
#define ITEM_LIGHT		      1
#define ITEM_SCROLL		      2
#define ITEM_WAND		         3
#define ITEM_STAFF		      4
#define ITEM_WEAPON		      5
#define ITEM_TICKET				6
#define ITEM_TREASURE		   8
#define ITEM_ARMOR		      9
#define ITEM_POTION		     10
#define ITEM_FURNITURE		  12
#define ITEM_TRASH		     13
#define ITEM_CONTAINER		  15
#define ITEM_DRINK_CON		  17
#define ITEM_KEY		        18
#define ITEM_FOOD		        19
#define ITEM_MONEY		     20
#define ITEM_BOAT		        22
#define ITEM_CORPSE_NPC		  23
#define ITEM_CORPSE_PC		  24
#define ITEM_FOUNTAIN		  25
#define ITEM_PILL		        26
#define ITEM_PORTAL			  28



// Extra flags.
// Used in #OBJECTS.
#define ITEM_GLOW		      1
#define ITEM_HUM		      2
#define ITEM_DARK		      4
#define ITEM_EVIL		     16
#define ITEM_INVIS		  32
#define ITEM_MAGIC		  64
#define ITEM_NODROP		 128
#define ITEM_BLESS		 256
#define ITEM_ANTI_GOOD	 512
#define ITEM_ANTI_EVIL	1024
#define ITEM_ANTI_NEUTRAL 2048
#define ITEM_NOREMOVE	4096
#define ITEM_INVENTORY	8192
#define ITEM_TIMER		16384
#define ITEM_GOOD			65536L
#define ITEM_NOLOCATE	131072L
#define ITEM_SHARP		524288L
#define ITEM_DULL			1048576L



// Wear flags.
// Used in #OBJECTS.
#define ITEM_TAKE		      1
#define ITEM_WEAR_FINGER	      2
#define ITEM_WEAR_NECK		      4
#define ITEM_WEAR_BODY		      8
#define ITEM_WEAR_HEAD		     16
#define ITEM_WEAR_LEGS		     32
#define ITEM_WEAR_FEET		     64
#define ITEM_WEAR_HANDS		    128
#define ITEM_WEAR_ARMS		    256
#define ITEM_WEAR_SHIELD	    512
#define ITEM_WEAR_ABOUT		   1024
#define ITEM_WEAR_WAIST		   2048
#define ITEM_WEAR_WRIST		   4096
#define ITEM_WIELD		      8192
#define ITEM_HOLD		        16384
// TFC only
#define ITEM_WORN_PRIDE      32768L


// Apply types (for affects).
// Used in #OBJECTS.
#define APPLY_NONE		      0
#define APPLY_STR		      1
#define APPLY_DEX		      2
#define APPLY_INT		      3
#define APPLY_WIS		      4
#define APPLY_CON		      5
#define APPLY_SEX		      6
#define APPLY_CLASS		      7
#define APPLY_LEVEL		      8
#define APPLY_AGE		      9
#define APPLY_HEIGHT		     10
#define APPLY_WEIGHT		     11
#define APPLY_MANA		     12
#define APPLY_HIT		     13
#define APPLY_MOVE		     14
#define APPLY_GOLD		     15
#define APPLY_EXP		     16
#define APPLY_AC		     17
#define APPLY_HITROLL		     18
#define APPLY_DAMROLL		     19
#define APPLY_SAVING_PARA	     20
#define APPLY_SAVING_ROD	     21
#define APPLY_SAVING_PETRI	     22
#define APPLY_SAVING_BREATH	     23
#define APPLY_SAVING_SPELL	     24
#define APPLY_SANCTUARY			  25
#define APPLY_FLYING				  26
#define APPLY_BLIND				  27
#define APPLY_INVISIBLE			  28
#define APPLY_DETECT_EVIL		  29
#define APPLY_DETECT_INVIS		  30
#define APPLY_DETECT_MAGIC		  31
#define APPLY_DETECT_HIDDEN     32
#define APPLY_FAERIE_FIRE		  33
#define APPLY_INFRARED			  34
#define APPLY_CURSE				  35
#define APPLY_POISON				  36
#define APPLY_PROTECT			  37
#define APPLY_SNEAK				  38
#define APPLY_HIDE				  39
#define APPLY_PASS_DOOR			  40
#define APPLY_PLAGUE			     41
#define APPLY_ENDURANCE			  42

// Values for containers (value[1]).
// Used in #OBJECTS.
#define CONT_CLOSEABLE		      1
#define CONT_PICKPROOF		      2
#define CONT_CLOSED		      4
#define CONT_LOCKED		      8


// Well known room virtual numbers.
#define ROOM_VNUM_TEMPLE	   3001

/*
 * Room flags.
 * Used in #ROOMS.
 */
#define ROOM_DARK		            1
#define ROOM_HOUSE					2  // AVATAR
#define ROOM_NO_MOB		         4
#define ROOM_INDOORS		         8
#define ROOM_NO_TRACK			  32  //AVATAR
#define ROOM_ARENA				  64 //AVATAR
#define ROOM_ANTI_MAGIC			 128 //AVATAR
#define ROOM_PRIVATE		       512
#define ROOM_SAFE		         1024
#define ROOM_SOLITARY		   2048
#define ROOM_PET_SHOP		   4096
#define ROOM_NO_RECALL		   8192
#define ROOM_BANK				  16384 //AVATAR
#define ROOM_SILENCED		  32768L //AVATAR
#define ROOM_INFIRMARY		  65536L //AVATAR
#define ROOM_SHADED			 131072L //AVATAR
#define ROOM_NOEMOTE			 262144L //AVATAR
#define ROOM_LLOYDABLE	   1048576L //AVATAR
#define ROOM_NOIMP			2097152L //AVATAR


// AVATAR
/*
 * Area flags.   NEW!!!
 * Used in #AREADATA.
 */

#define AREA_NO_FLAG 		0  // not needed
#define AREA_NO_PLAGUE		1  // plague won't spread here
#define AREA_NO_SUMMON		2  // things cannot be summoned here
#define AREA_ARENA			4  // every room in area is arena

// Planes

#define PLANE_MIDGAARDIA	1          //default
#define PLANE_OUTLAND		2
#define PLANE_ARENA			3
#define PLANE_IMMORTAL		4
#define PLANE_HELL			5
#define PLANE_TARTERUS		6
#define PLANE_STONE			7
#define PLANE_THORNGATE		8
#define PLANE_ARCADIA		9
#define PLANE_ASTRAL			10



// Directions.
// Used in #ROOMS.
#define DIR_NORTH		      0
#define DIR_EAST		      1
#define DIR_SOUTH		      2
#define DIR_WEST		      3
#define DIR_UP			      4
#define DIR_DOWN		      5



// Exit flags.
// Used in #ROOMS.
#define EX_ISDOOR		      1
#define EX_CLOSED		      2
#define EX_LOCKED		      4
#define EX_PICKPROOF		     32
#define EX_BASHED                     8
#define EX_BASHPROOF                 16
#define EX_PASSPROOF                 64

// Sector types.
// Used in #ROOMS.
#define SECT_INSIDE		      0
#define SECT_CITY		      1
#define SECT_FIELD		      2
#define SECT_FOREST		      3
#define SECT_HILLS		      4
#define SECT_MOUNTAIN		      5
#define SECT_WATER_SWIM		      6
#define SECT_WATER_NOSWIM	      7
#define SECT_AIR		      9
#define SECT_DESERT		     10
#define SECT_UNDERWATER   11
#define SECT_ON_BOTTOM	  12
#define SECT_MAX		     13



// Equpiment wear locations.
// Used in #RESETS.
#define WEAR_NONE		        -1
#define WEAR_LIGHT		      0
#define WEAR_FINGER_L		   1
#define WEAR_FINGER_R		   2
#define WEAR_NECK_1		      3
#define WEAR_NECK_2		      4
#define WEAR_BODY		         5
#define WEAR_HEAD		         6
#define WEAR_LEGS		         7
#define WEAR_FEET		         8
#define WEAR_HANDS		      9
#define WEAR_ARMS		        10
#define WEAR_SHIELD		     11
#define WEAR_ABOUT		     12
#define WEAR_WAIST		     13
#define WEAR_WRIST_L		     14
#define WEAR_WRIST_R		     15
#define WEAR_WIELD		     16
#define WEAR_HOLD		        17
#define MAX_WEAR             18
// TFC-only
#define WEAR_PRIDE	        18 /* Also called WEAR_DECOR */
#define WEAR_DECOR           18
//#define MAX_WEAR		     19

#define UPPER( c ) ((char)toupper(c))
//#define UPPER( c )		( ( c ) >= 'a' && ( c ) <= 'z' ? ( c ) + 'A' - 'a' : ( c ) )

#define LOWER( c ) ((char)tolower(c))
//#define LOWER( c )		( ( c ) >= 'A' && ( c ) <= 'Z' ? ( c ) + 'a' - 'A' : ( c ) )

#define IS_SET( flag, bit )	( (( flag ) &   ( bit )) == ( bit ) )
#define SET_BIT( var, bit )	( ( var )  |=  ( bit ) )
#define REMOVE_BIT( var, bit )	( ( var )  &= ~( bit ) )
#define IS_GOOD( ch )		( ch->alignment >=  350 )
#define IS_EVIL( ch )		( ch->alignment <= -350 )
//#define SET_RESET_BIT(orig,flg,toggle) { ((flg) ? SET_BIT(orig, toggle) : REMOVE_BIT(orig, toggle)) }
#define SET_RESET_BIT(orig,flg,toggle) { if (flg) SET_BIT(orig,toggle); else REMOVE_BIT(orig,toggle); }
//#define SET_RESET_BIT(orig, flg, toggle) ((flg) ? (orig) : (toggle))

/* dice */
#define DICE_NUMBER			0
#define DICE_TYPE			1
#define DICE_BONUS			2

/* ROM AC types */
#define AC_PIERCE			0
#define AC_BASH				1
#define AC_SLASH			2
#define AC_EXOTIC			3

class TVNumObj: public TMercObj {
public:
    TVNumObj(TAreaDocument *doc);
    TVNumObj(const TVNumObj& other);
    int operator==(const TVNumObj& other) const { return vnum == other.vnum; }

    sh_int vnum;
    sh_int new_vnum;
};

class TMobProgData: public TMercObj {
public:
    TMobProgData(TAreaDocument *doc);
    TMobProgData(TMobProgData& other);
    const virtual TMercObj *GetName(char *lpOut, int nOutSize) const;
    int operator==(const TMobProgData& other) const;
    const TMobProgData& operator=(const TMobProgData& other);
    static void DeleteAMobProg(TMobProgData *& p, void *);

//    MPROG_DATA *next;
//    int         type;
    string      type;
    string      arglist;
    string      comlist;
    string      MobFile;   // Filename of MOBprogam
};
typedef TMercPointer<TMobProgData> PMobProgData;

// Prototype for a mob.
// This is the in-memory version of #MOBILES.
class TMobile: public TVNumObj {
public:
    TMobile(TAreaDocument *);
    TMobile(const TMobile&);
    ~TMobile();
    const virtual TVNumObj *GetName(char *lpOut, int nOutSize) const;
	 char HUGE *MobileToText(char HUGE *) const;
    const TMobile& operator=(const TMobile&);
    BOOL is_affected(int32 sn) const { return IS_SET( affected_by, sn ); }
    int operator==(const TMobile& other) const;

    string     spec_fun;
    string     spec_comment;
	 PShopData	pShop;
	 string		player_name;
	 string		short_descr;
	 string		long_descr;
	 string		description;
//	 sh_int 		vnum;
//	 int32 		count;
//	 int32 		killed;
	 sh_int 		sex;
	 sh_int 		level;
	 int32		act;
	 int32		affected_by;
	 sh_int 		alignment;

    // The following are unused by Merc, we read and write them however
	 int32 		hitroll;
	 int32 		ac;
	 int32 		hitnodice;
	 int32 		hitsizedice;
	 int32 		hitplus;
	 int32 		damnodice;
	 int32 		damsizedice;
	 int32 		damplus;
	 int32		gold;

    TArrayAsVector<TMobProgData*> mobprogs;

    // ROM bits
    sh_int		mana[3];
    sh_int		rom_ac[4];
    sh_int 		dam_type;
    int32		off_flags;
    int32		imm_flags;
    int32		res_flags;
    int32		vuln_flags;
    sh_int		start_pos;
    sh_int		default_pos;
    sh_int		race;
    sh_int		clss;
    int32	   form;
    int32	   parts;
    sh_int		size;
//    sh_int		material;
    string     material;
};
typedef TMercPointer<TMobile> PMobile;

// Extra description data for a room or object.
class TExtraDescr /*: public TMercObj*/ {
public:
//    TExtraDescr(TAreaDocument *Doc);
    TExtraDescr(const TExtraDescr& other);
    TExtraDescr() {}
    int operator==(const TExtraDescr& other) const;

    string keyword;     /* Keyword in look/examine          */
    string description; /* What to see                      */
};
typedef TMercPointer<TExtraDescr> PExtraDescr;


// Prototype for an object.
class TObject: public TVNumObj {
public:
    TObject(TAreaDocument *Doc);
    TObject(const TObject&);
    ~TObject();
    const TObject& operator=(const TObject& other);
    int operator==(const TObject& other) const;

    const /*virtual*/ TVNumObj *GetName(char *lpOut, int nOutSize) const;
	 char HUGE *ObjToText(char HUGE *p) const;
    BOOL is_obj_stat(int32 stat) const { return IS_SET(extra_flags, stat); }

//    TListImp<TExtraDescr*> extra_descr;
    TArrayAsVector<TExtraDescr*> extra_descr;
//    TListImp<TAffect*> affected;
    TArrayAsVector<TAffect*> affected;
    string		name;
    string		short_descr;
    string		description;
    sh_int		item_type;
    int32 		extra_flags;
    int32 		wear_flags;
    int32 		count;
    int32 		weight;        // sh_int in Merc, int in Envy
    int32		cost;			/* Unused by Merc */
    string     value[5];   // Value[4] only used by ROM

    // ROM
    sh_int 		condition;
	 string     material;
	 sh_int     level;
};
typedef TMercPointer<TObject> PObject;

// Exit data.
class TExit: public TMercObj {
public:
	 TExit(TAreaDocument *doc=0);
	 int operator==(const TExit& other) const;

	 TRoom     *to_room;
	 sh_int 		vnum;
	 sh_int 		exit_info;
	 sh_int 		key;
	 string		keyword;
	 string		description;

	 TReset *InitialPosition;
	 int nInitialPosition; // 0 = open, 1 = closed, 2 = locked, 3 = leave alone
};
typedef TMercPointer<TExit> PExit;


/*
 * Reset commands:
 *   '*': comment
 *   'M': read a mobile
 *   'O': read an object
 *   'P': put object in object
 *   'G': give object to mobile
 *   'E': equip object to mobile
 *   'D': set state of door
 *   'R': randomize room exits
 *   'S': stop (end of list)
 */

// Area-reset definition.
class TReset: public TMercObj {
public:
    TReset(TAreaDocument *doc);
    TReset(const TReset&);
    const virtual TMercObj *GetName(char *lpOut, int nOutSize) const;
    int operator==(const TReset& other) const;
    const TReset& operator=(const TReset&);

    char		command;
    sh_int 		arg1;
    sh_int 		arg2;
    sh_int 		arg3;
    string  comment;
};
typedef TMercPointer<TReset> PReset;

class TMobReset: public TReset {
public:
    TMobReset(TAreaDocument *doc);
    TMobReset(const TMobReset&);
    const virtual TMercObj *GetName(char *lpOut, int nOutSize) const;
    int operator==(const TMobReset& other) const;
    const TMobReset& operator=(const TMobReset&);

	 TArrayAsVector<TReset *> Wields;
    TArrayAsVector<TReset *> Inventory;
};
typedef TMercPointer<TMobReset> PMobReset;

// Room type.
class TRoom: public TVNumObj {
public:
    TRoom(TAreaDocument *doc);
    TRoom(const TRoom& other);
    ~TRoom();
    const TRoom& operator=(const TRoom& other);
    int operator==(const TRoom& other) const;
    const virtual TVNumObj *GetName(char *lpOut, int nOutSize) const;
    char HUGE *RoomToText(char HUGE *) const;

	 TArrayAsVector<TExtraDescr*> extra_descr;
	 TExit      exit[6];
	 BOOL       is_exit[6];
	 string		name;
	 string		description;
	 int32 		room_flags;
	 int 		sector_type;

	 TArrayAsVector<TMobReset *> MobResets;
	 TArrayAsVector<TReset *> ObjResets;
	 TReset *RandomDoor;
};
typedef TMercPointer<TRoom> PRoom;

#if 0
/* Aff2 bits.  Can not be used to #MOBILES, since aff2 is not part of
	Diku structure. TFC only. */

#define AF2_ATTEMPT_BLYND   1
#define AF2_NOSEXCHANGE     2
#define AF2_NOINFO          4
#define AF2_DEAFNESS        8
#define AF2_SILENCE        16
#define AF2_WIZARD_MARK    32
#define AF2_FORGET         64
#define AF2_FEEBLEMIND    128
#define AF2_MINDBLANK     256
#define AF2_COMPREHEND    512
#define AF2_OBSCURE      1024
#define AF2_DISEASE      2048
#define AF2_CONFUSION    4096
#define AF2_TONGUES      8192
#define AF2_ONSET       16384
#endif

#if 1
/*
 * Area definition. (Troll only)
 */
typedef struct	area_weather		AREA_WEATHER;

struct	area_weather
{
	int		MinTemp;
	int		MaxTemp;
	int		MinPrecip;
	int		MaxPrecip;
	int		MinWind;
	int		MaxWind;
	int		MinMagic;
	int		MaxMagic;
	int		MaxChange;
};

class TAreaData
{
public:
    BOOL fCustomWeather;
    AREA_WEATHER	Weather;	/* For NEW Weather System.  James. */
//    char *		name; // (Stored in TAreaDocument)
    string		repop;		/* From The Isles code.  Walker */
    int		area_flags;	/* Support area flags.  Walker */
    sh_int		plane;
//    int     	recall;   // (Stored in TAreaDocument)
    int			timezone;	/* For NEW Weather System.  James. */
};
#endif

#endif
